MODULE guess_form
!
!**********************************************************************************
!*  GUESS_FORM                                                                    *
!**********************************************************************************
!* This module tries to determine the format of a file.                           *
!* The guess is made on the contents of the file, rather than                     *
!* on its extension. If the file doesn't exist, then only the                     *
!* extension is taken into account.                                               *
!**********************************************************************************
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 28 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
! NOTE:
!This algorithm first checks the file extension, but does not
!fully trust it to determine the file format: it parses the file
!and searches for some specific patterns.
!Each recognized pattern makes it more and more sure that
!the input file corresponds to a specific format.
!
USE comv
USE messages
USE files
USE subroutines
!
!
CONTAINS

SUBROUTINE GUESS_FORMAT(inputfile,infileformat)
!
IMPLICIT NONE
CHARACTER(LEN=3):: extension, infileformat
CHARACTER(LEN=64):: test
CHARACTER(LEN=64):: inputfile
CHARACTER(LEN=128):: msg
LOGICAL:: fileexists
INTEGER:: length, i
INTEGER:: NP
REAL*8:: isxyz, iscfg, isgin, isxsf, iscml, isposcar
REAL*8:: likely
REAL*8:: smass
REAL*8:: testreal
REAL*8:: certainty
!
!
!Initialize variables
 certainty = 0.9d0
infileformat = ''
extension = ''
test=''
i = 1
testreal = 0.d0
isxyz = 0.d0
iscfg = 0.d0
isxsf = 0.d0
isgin = 0.d0
iscml = 0.d0
isposcar = 0.d0
!
msg = 'debug --> Entering GUESS_FORMAT...'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
INQUIRE(FILE=inputfile,EXIST=fileexists)
IF(fileexists) THEN
  certainty = 0.8d0
  OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='UNKNOWN',POSITION='REWIND')
ELSE
  certainty = 0.2d0
ENDIF
!
!
100 CONTINUE
msg = 'debug --> Check extension of the file: '//TRIM(inputfile)
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!First, check the extension
length = LEN_TRIM(inputfile)
IF(inputfile(length-3:length-3)=='.') THEN
  extension = inputfile(length-2:length)
  msg = 'debug --> Extension: '//extension
ELSE
  extension = ''
  msg = 'debug --> This file has no extension: '//inputfile
ENDIF
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
SELECT CASE(extension)
CASE('xyz')
  isxyz = isxyz+0.6d0
CASE('cfg')
  iscfg = iscfg+0.4d0
CASE('xsf')
  isxsf = isxsf+0.4d0
CASE('gin','res','grs')
  isgin = isgin+0.4d0
CASE('cml')
  iscml = iscml+0.4d0
CASE('CAR')
  isposcar = isposcar+0.1d0
CASE DEFAULT
  CONTINUE
END SELECT
IF(TRIM(ADJUSTL(inputfile))=='POSCAR') isposcar = certainty-0.15d0
!
!
200 CONTINUE
msg = 'debug --> Parsing file content...'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
IF(fileexists) THEN
!Try to confirm the format by reading the contents of the file
!We limit it to the first 10,000 lines to avoid loosing too much time
DO i=1,10000
  READ(30,'(a64)',END=300,ERR=300) test
  test = TRIM(ADJUSTL(test))
  !
  IF(i<100) THEN
    msg = 'debug --> GUESS_FORMAT line:  '//TRIM(test)
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  ELSEIF(i==100) THEN
    msg = 'debug --> GUESS_FORMAT line:  ... discontinued ...'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  ENDIF
  !
  IF(test(1:1).NE.'#') THEN
    !
    !Search for patterns corresponding to CFG format
    IF(test(1:19)=='Number of particles') THEN
      iscfg = iscfg+0.3d0
    ELSEIF(test(1:3)=='A =' .OR. test(1:2)=='A=') THEN
      iscfg = iscfg+0.2d0
    ELSEIF(test(1:6)=='H(1,1)' .OR. test(1:6)=='H(1,2)' .OR. &
          &test(1:6)=='H(2,1)' .OR. test(1:6)=='H(2,2)' .OR. &
          &test(1:6)=='H(3,1)' .OR. test(1:6)=='H(3,2)' .OR. &
          &test(1:6)=='H(3,3)') THEN
      iscfg = iscfg+0.1
    ELSEIF(test(1:11)=='entry_count') THEN
      iscfg = iscfg+0.3d0
    ELSEIF(test(1:9)=='auxiliary') THEN
      iscfg = iscfg+0.2d0
    !
    !Search for patterns corresponding to GIN format
    ELSEIF(test(1:5)=='title') THEN
      isgin = isgin+0.2d0
    ELSEIF(test(1:4)=='cell') THEN
      isgin = isgin+0.2d0
    ELSEIF(test(1:4)=='vect') THEN
      isgin = isgin+0.1d0
    ELSEIF(test(1:3)=='car') THEN
      isgin = isgin+0.1d0
    ELSEIF(test(1:4)=='buck') THEN
      isgin = isgin+0.3d0
    ELSEIF(test(1:7)=='spieces') THEN
      isgin = isgin+0.2d0
    ELSEIF(test(1:6)=='spring') THEN
      isgin = isgin+0.2d0
    ELSEIF(test(1:3)=='fra') THEN
      isgin = isgin+0.1d0
    ELSEIF(test(1:7)=='keyword') THEN
      isgin = isgin+0.3d0
    ELSEIF(test(1:4)=='cutp') THEN
      isgin = isgin+0.2d0
    ELSEIF(test(4:7)=='core') THEN
      isgin = isgin+0.3d0
    ELSEIF(test(4:7)=='shel') THEN
      isgin = isgin+0.3d0
    ELSEIF(test(1:5)=='total') THEN
      isgin = isgin+0.1d0
    ELSEIF(test(1:10)=='dump every') THEN
      isgin = isgin+0.2d0
    ELSEIF(test(1:4)=='output') THEN
      isgin = isgin+0.1d0
    !
    !Search for patterns corresponding to XSF format
    ELSEIF(test=='PRIMCOORD') THEN
      isxsf = isxsf+0.3d0
    ELSEIF(test=='PRIMVEC') THEN
      isxsf = isxsf+0.3d0
    ELSEIF(test=='CONVVEC') THEN
      isxsf = isxsf+0.3d0
    !
    !Search for patterns corresponding to CML format
    !ELSEIF(test(1,1)=='<') THEN
    !  iscml = iscml+0.1d0
    !
    !Search for patterns corresponding to VASP POSCAR format
    ELSEIF(test(1:18)=='Selective dynamics') THEN
      isposcar = isposcar+0.6d0
    ELSEIF(test(1:6)=='Direct') THEN
      isposcar = isposcar+0.3d0
    ELSEIF(test(1:9)=='Cartesian') THEN
      isposcar = isposcar+0.3d0
    !
    !Search for patterns corresponding to XYZ format
    ELSEIF(test(1:4)=='alat') THEN
      isxyz = isxyz+0.2d0
    ELSEIF(test(1:5)=='super') THEN
      isxyz = isxyz+0.2d0
    ELSEIF(i==1) THEN
      READ(test,*,END=190,ERR=190) NP
      isxyz = isxyz+0.1d0
    ELSEIF(test(3:3)=='') THEN
      CALL ATOMMASS(test(1:2),smass)
      IF(smass.NE.0.d0) THEN
        READ(test(3:64),*,ERR=150,END=150) testreal, testreal, testreal
        isxyz = isxyz+0.01d0
        150 CONTINUE
      ENDIF
    !
    ! -- Add other formats here --
    ENDIF
  !
  ENDIF
  !
  190 CONTINUE
!
ENDDO
!
!
ENDIF   !If fileexists
!
!
300 CONTINUE
CLOSE(30)
!
!Find the best score
likely = MAX(isxyz,iscfg,isxsf,isgin,isposcar)
!
WRITE(msg,*) 'debug --> Score of file format: ', likely
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
IF(likely<certainty/2.d0) THEN
  !If the best score is very low then it is probably
  !an unknown file format
  infileformat = 'xxx'
!
!Otherwise we have detected a known file format
!(GF_WARN subroutine is found below)
ELSEIF(isxyz==likely) THEN
  infileformat = 'xyz'
  IF(isxyz<certainty) CALL GF_WARN(infileformat)
ELSEIF(iscfg==likely) THEN
  infileformat = 'cfg'
  IF(iscfg<certainty) CALL GF_WARN(infileformat)
ELSEIF(isgin==likely) THEN
  infileformat = 'gin'
  IF(isgin<certainty) CALL GF_WARN(infileformat)
ELSEIF(isxsf==likely) THEN
  infileformat = 'xsf'
  IF(isxsf<certainty) CALL GF_WARN(infileformat)
ELSEIF(iscml==likely) THEN
  infileformat = 'cml'
  IF(iscml<certainty) CALL GF_WARN(infileformat)
ELSEIF(isposcar==likely) THEN
  infileformat = 'pos'
  IF(isposcar<certainty) CALL GF_WARN(infileformat)
! -- add other formats here --
ELSE
  infileformat = 'xxx'
ENDIF
!
WRITE(msg,*) 'debug --> File format: ', infileformat
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
500 CONTINUE
!
!
END SUBROUTINE GUESS_FORMAT
!
!
!
!**************************************************
! GF_WARN
! This subroutine just displays a warning message
! for the GUESS_FORMAT routine
!**************************************************
SUBROUTINE GF_WARN(infileformat)
!
CHARACTER(LEN=3):: infileformat
CHARACTER(LEN=128):: msg
!
nwarn = nwarn+1
msg = '/!\ WARNING: the most probable file format was found to be '//infileformat
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '    However the format could not be determined with certainty.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
END SUBROUTINE GF_WARN
!**************************************************
!
!
!
END MODULE guess_form
