MODULE in_gulp_gin
!
!**********************************************************************************
!*  IN_GULP_GIN                                                                   *
!**********************************************************************************
!* This module reads GULP input files (.gin) and restart files (.res or .grs).    *
!* This format is described in the GULP manual, which can be found here:          *
!*    https://www.ivec.org/gulp/help/manuals.html                                 *
!**********************************************************************************
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 17 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE functions
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
SUBROUTINE READ_GIN(inputfile)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!RESTRICTIONS OF THIS ROUTINE:
!1- in the input file (.gin, .res or .grs), there must be
!   NO COMMENT LINE following a 'vectors' or 'cell' statement!
!   For instance this would NOT be correct:
!          +----------------------------------------+
!          | cell                                   |
!          | # This comment is in the wrong place   |
!          | 4.02 4.02 4.02 90.0 90.0 90.0          |
!          +----------------------------------------+
!   In that case the line starting with # will cause an error,
!   it has to be placed elsewhere.
!   Apart from that, comments (i.e., lines starting with #) in
!   the input file should not cause any problem.
!
!2- Symmetry operations are not taken into account. So, if
!   you specify coordinates in a primitive lattice, only
!   atoms in that primitive cell will be read and output.
!   Symmetry is a whole other story to implement, so it was
!   not done in this program.
!
!3- since NEB was implemented in GULP (>3.4), the RES file
!   can contain all atomic positions for all images of NEB.
!   This is not recognized here, and only the positions
!   of the 1st NEB image will be read and converted by
!   this program.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
IMPLICIT NONE
CHARACTER(LEN=2):: spieces
CHARACTER(LEN=4):: coord
CHARACTER(LEN=9):: coord1, coord2, coord3
CHARACTER(LEN=32):: xyzfile
CHARACTER(LEN=64):: inputfile
CHARACTER(LEN=64):: test, test2
CHARACTER(LEN=128):: comment, msg
INTEGER:: i
INTEGER:: icoordtemp1, icoordtemp3
INTEGER:: NP
REAL*8:: alpha, beta, gamma
REAL*8:: Hx, Hy, Hz !Length of H(1,:), H(2,:) and H(3,:)
REAL*8:: smass !Mass of atoms
REAL*8, DIMENSION(3):: P
!
!
!Initialize variables
 comment='File generated by pikashu'
test=''
NP=100000
P(:) = 0.d0
H(:,:) = 0.d0
!
!
10 CONTINUE
xyzfile = 'pikashu.xyz'
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!
!
20 CONTINUE
!Find and read the supercell vectors
READ(30,*,ERR=600,END=600) test
test = TRIM(ADJUSTL(test))
IF(test(1:5)=='title') THEN
  READ(30,'(a64)',ERR=600,END=600) test(1:64)
  !We take the first line as "comment" and ignore the rest
  comment=TRIM(ADJUSTL(test))
  DO WHILE(test(1:3).NE.'end')
    READ(30,*,ERR=600,END=600) test
    test = TRIM(ADJUSTL(test))
  ENDDO
  GOTO 20
ELSEIF(test(1:3)=='vec') THEN
  msg = 'debug --> Found cartesian vectors.'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  READ(30,*,END=600,ERR=600) H(1,1), H(1,2), H(1,3)
  READ(30,*,END=600,ERR=600) H(2,1), H(2,2), H(2,3)
  READ(30,*,END=600,ERR=600) H(3,1), H(3,2), H(3,3)
  GOTO 25
ELSEIF(test(1:3)=='cel') THEN
  msg = 'debug --> Found conventional vectors:'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  READ(30,*,END=600,ERR=600) Hx, Hy, Hz,             &
      &                      alpha, beta, gamma
  WRITE(msg,21) 'debug -->   a = ', Hx, '     alpha = ', alpha
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  WRITE(msg,21) 'debug -->   b = ', Hy, '      beta = ', beta
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  WRITE(msg,21) 'debug -->   c = ', Hz, '     gamma = ', gamma
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  !In GULP angles are in degrees: convert to radians
  alpha = alpha*pi/180.d0
  beta  = beta*pi/180.d0
  gamma = gamma*pi/180.d0
  !Convert basis vectors in cartesian coordinates
  CALL CONVMAT(Hx,Hy,Hz,alpha,beta,gamma,H)
  GOTO 25
ELSE
  GOTO 20
ENDIF
21 FORMAT(a16,f9.5,a13,f9.5)
!
!
25 CONTINUE
msg = 'debug --> Base vectors:'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,27) 'debug -->      | ', H(1,1), H(1,2), H(1,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,27) 'debug -->  H = | ', H(2,1), H(2,2), H(2,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,27) 'debug -->      | ', H(3,1), H(3,2), H(3,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
Hset = .TRUE.
!
!Return to beginning of file, because sometimes
!the vectors are written after the coordinates
REWIND(30)
!
26 CONTINUE
!Find out if coordinates are fractional or cartesian
READ(30,*,ERR=600,END=600) test
IF(test(1:3)=='fra') THEN
  msg = 'debug --> Found fractional coordinates.'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  coord='frac'
  GOTO 30
ELSEIF(test(1:3)=='car') THEN
  msg = 'debug --> Found cartesian coordinates.'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  coord='cart'
  GOTO 30
ELSE
  GOTO 26
ENDIF
27 FORMAT(a11,3(f10.5,2X),a1)
!
!
29 CONTINUE
msg = '/!\ System information was not found!'
CALL DISPLAY_MSG(3,msg,logfile)
msg = '(°< Enter cell dimensions Hx, Hy, Hz (angstroms):'
CALL DISPLAY_MSG(3,msg,logfile)
READ(*,*) Hx, Hy, Hz
msg = '(°< Enter the angles alpha, beta, gamma (degrees):'
CALL DISPLAY_MSG(3,msg,logfile)
READ(*,*) alpha, beta, gamma
!Convert to radians
alpha = alpha*pi/180.d0
beta  = beta*pi/180.d0
gamma = gamma*pi/180.d0
!Convert basis vectors into cartesian coordinates
CALL CONVMAT(Hx,Hy,Hz,alpha,beta,gamma,H)
msg = '(°< Enter number of particles:'
CALL DISPLAY_MSG(3,msg,logfile)
READ(*,*) NP
!
!
30 CONTINUE
NP=0
!Count the total number of atoms NP
DO i=1,10000000  !Can go up to 10million atoms,increase it if needed
  READ(30,*,ERR=31,END=31), test, test2
  IF(test(1:1).NE.'#') THEN
    !We dont want to count the shells
    IF(TRIM(ADJUSTL(test2))=='core') NP=NP+1
    !If there is no 'shel' or 'core' on the line,
    !then we are done counting
    IF( TRIM(ADJUSTL(test2)).NE.'core' .AND.                &
      & TRIM(ADJUSTL(test2)).NE.'shel'       ) GOTO 31
  ENDIF
ENDDO
!
31 CONTINUE
WRITE(msg,'(a24,i8)') 'debug --> Number of particles:', NP
CALL DISPLAY_MSG(verbosity,msg,logfile)
REWIND(30)
DO i=1,100000
  READ(30,*,ERR=31), test
  IF( test(1:1).NE.'#' .AND.                                &
    & (test(1:3)=='fra' .OR. test(1:3)=='car') ) GOTO 40
ENDDO
!
!
40 CONTINUE
!Read and write atomic positions
spieces = ''
!'extended.xyz' is a temporary file that will contain
! information for the extended XYZ format
OPEN(UNIT=32,FILE='extended.xyz',FORM='FORMATTED',STATUS='UNKNOWN')
OPEN(UNIT=35,FILE='pikashu.xyz',FORM='FORMATTED',STATUS='UNKNOWN')
WRITE(35,*) NP
WRITE(35,*) TRIM(ADJUSTL(comment))
!
DO i=1,NP
  READ(30,*,ERR=600,END=600) test, test2, coord1, coord2, coord3
  coord1 = TRIM(ADJUSTL(coord1))
  coord2 = TRIM(ADJUSTL(coord2))
  coord3 = TRIM(ADJUSTL(coord3))
  !
  !If it is a comment then skip the whole thing and read next line
  IF(test(1:1).NE.'#' .AND. LEN_TRIM(test).NE.0) THEN
    !Test if it is a new atomic spieces
    IF(TRIM(ADJUSTL(test)).NE.spieces) THEN
      spieces=TRIM(ADJUSTL(test))
      !
      CALL ATOMMASS(spieces,smass)
      IF(smass==0.d0) GOTO 500
      !
      !Write info about atomic spieces in a temp file
      WRITE(32,'(a5,a2,2X,f12.6)') 'mass ', spieces, smass
    ENDIF
    !
    !Get rid of shells in case of shell models
    IF(TRIM(ADJUSTR(test2))=='shel') GOTO 49
    !
    !GULP sometimes output the numbers as fractions,
    !eg. 1/3 or 1/5, which Fortran does not consider
    !as REAL... so we have to convert that...
    IF( LEN_TRIM(coord1)==3 .OR. coord1(2:2)=='/' ) THEN
      READ(coord1(1:1),*) icoordtemp1
      READ(coord1(3:3),*) icoordtemp3
      P(1)=REAL(icoordtemp1)/REAL(icoordtemp3)
    ELSE
      READ(coord1,*) P(1)
    ENDIF
    IF( LEN_TRIM(coord2)==3 .OR. coord2(2:2)=='/' ) THEN
      READ(coord2(1:1),*) icoordtemp1
      READ(coord2(3:3),*) icoordtemp3
      P(2)=REAL(icoordtemp1)/REAL(icoordtemp3)
    ELSE
      READ(coord2,*) P(2)
    ENDIF
    IF( LEN_TRIM(coord3)==3 .OR. coord3(2:2)=='/' ) THEN
      READ(coord3(1:1),*) icoordtemp1
      READ(coord3(3:3),*) icoordtemp3
      P(3)=REAL(icoordtemp1)/REAL(icoordtemp3)
    ELSE
      READ(coord3,*) P(3)
    ENDIF
    !
    !Finally we can output atomic positions in XYZ
    IF(coord=='cart') THEN
      WRITE(35,105) smass, P(1), P(2), P(3)
    ELSEIF(coord=='frac') THEN
      WRITE(35,105) smass,                                      &
           &               P(1)*H(1,1)+P(2)*H(2,1)+P(3)*H(3,1), &
           &               P(1)*H(1,2)+P(2)*H(2,2)+P(3)*H(3,2), &
           &               P(1)*H(1,3)+P(2)*H(2,3)+P(3)*H(3,3)
    ENDIF
  !
  ENDIF
  49 CONTINUE
ENDDO
!
100  FORMAT(3(f10.6,2x)) 
105  FORMAT(f6.2,X,3(f10.6,2x))
!
!
500 CONTINUE
!
!Add extended info at the end of XYZ file
WRITE(35,'(a4)') 'alat'
WRITE(35,'(a3)') '1.0'
WRITE(35,'(a9)') 'supercell'
WRITE(35,'(3(f12.6,X))') H(1,1), H(1,2), H(1,3)
WRITE(35,'(3(f12.6,X))') H(2,1), H(2,2), H(2,3)
WRITE(35,'(3(f12.6,X))') H(3,1), H(3,2), H(3,3)
REWIND(32)
DO i=1,100
  READ(32,'(a64)',END=550) test
  WRITE(35,*) TRIM(ADJUSTL(test))
ENDDO
550 CONTINUE
WRITE(35,'(a21)') 'cartesian coordinates'
msg = 'debug --> I wrote the XYZ file: '//TRIM(xyzfile)
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
GOTO 1000
!
!
600 CONTINUE
msg = '/!\ ERROR: that input file has a bad format:'//TRIM(inputfile)
CALL DISPLAY_MSG(3,msg,logfile)
msg = '    Program will EXIT.'
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
1000 CONTINUE
CLOSE(30)
CLOSE(32)
CLOSE(35)
!Remove the temporary file
CALL SYSTEM('rm -f extended.xyz')
!
END SUBROUTINE READ_GIN
!
!
END MODULE in_gulp_gin
