MODULE in_vasp_poscar
!
!
!**********************************************************************************
!*  IN_VASP_POSCAR                                                                *
!**********************************************************************************
!* This module reads POSCAR files, as well as CONTCAR files since it              *
!* is the same format. These files are used by VASP.                              *
!* The POSCAR format is described here:                                           *
!*    http://cms.mpi.univie.ac.at/vasp/vasp/node61.html                           *
!* Beware that atomic spieces are not specified in the POSCAR format, hence       *
!* this module will use dummy atomic masses (1, 2, 3...) when converting          *
!* from POSCAR to another format.                                                 *
!**********************************************************************************
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 05 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
IMPLICIT NONE
CHARACTER(LEN=64):: inputfile
!
!
CONTAINS
!
SUBROUTINE READ_POSCAR(inputfile)
!
CHARACTER(LEN=4):: coord
CHARACTER(LEN=64):: inputfile
CHARACTER(LEN=64):: comment
CHARACTER(LEN=128):: msg, test
INTEGER:: i, j, k
INTEGER:: NP
INTEGER,DIMENSION(20):: NPi
REAL*8:: a0
REAL*8,DIMENSION(4):: P
!
!
!Initialize variables
NP = 0
NPi(:) = 0
a0 = 1.d0
P(:) = 0.d0
 coord = 'cart'
 comment = 'File generated by pikashu'
!
!
100 CONTINUE
WRITE(msg,*) 'debug --> entering READ_POSCAR'
CALL DISPLAY_MSG(verbosity,msg,logfile)
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD')
OPEN(UNIT=35,FILE='pikashu.xyz',FORM='FORMATTED',STATUS='UNKNOWN')
!
READ(30,'(a64)') comment
READ(30,*) a0
READ(30,*) H(1,1), H(1,2), H(1,3)
READ(30,*) H(2,1), H(2,2), H(2,3)
READ(30,*) H(3,1), H(3,2), H(3,3)
H(:,:) = a0*H(:,:)
!
!Read the different spieces
READ(30,'(a128)') test
DO k=1,20
  READ(test,*,ERR=110,END=110) (NPi(j), j=1,k)
ENDDO
!
!
110 CONTINUE
WRITE(msg,*) "debug --> Number of different spieces: ", k
CALL DISPLAY_MSG(verbosity,msg,logfile)
DO WHILE(test(1:4).NE.'Cart' .OR. test(1:6).NE.'Direct')
  READ(30,*,ERR=400,END=400) test
  test = TRIM(ADJUSTL(test))
ENDDO
IF(test(1:6)=='Direct') coord='frac'
!
!
!
200 CONTINUE
DO j=1,20
  NP = NP+NPi(j)
ENDDO
!Write header of pikashu.xyz
WRITE(35,*) NP
WRITE(35,*) TRIM(ADJUSTL(comment))
!
!Read atomic coordinates and write them to pikashu.xyz
P(4) = 0.d0
DO j=1,20
  DO i=1,NPi(j)
    P(4) = P(4)+1
    READ(30,*) P(1), P(2), P(3)
    IF(coord=='frac') THEN
      P(1) = P(1)*H(1,1)
      P(2) = P(2)*H(2,2)
      P(3) = P(3)*H(3,3)
    ENDIF
    WRITE(35,220) P(4), P(1), P(2), P(3)
  ENDDO
ENDDO
220 FORMAT(f6.2,X,3(f10.6,2x))
!
!
!
300 CONTINUE
!Write extended info at end of pikashu.xyz
WRITE(35,'(a4)') 'alat'
WRITE(35,'(f12.6)') '1.0'
WRITE(35,'(a9)') 'supercell'
WRITE(35,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(35,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(35,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
Hset = .TRUE.
GOTO 500
!
!
!
400 CONTINUE
nerr = nerr+1
msg = 'X!X ERROR: there was an error while parsing the file '//TRIM(inputfile)
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
!
500 CONTINUE
CLOSE(30)
CLOSE(35)
msg = "..> Successfully read POSCAR file: "//inputfile
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
END SUBROUTINE READ_POSCAR
!
END MODULE in_vasp_poscar
