MODULE in_xyz
!
!**********************************************************************************
!*  IN_XYZ                                                                        *
!**********************************************************************************
!* This module reads files in the XYZ format (normal or extended).                *
!* The XYZ format has never been formally specified nor                           *
!* documented; however the informal and widely used format                        *
!* is described for instance at:                                                  *
!*     http://openbabel.org/wiki/XYZ_%28format%29                                 *
!**********************************************************************************
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 12 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
SUBROUTINE READ_XYZ(inputfile)
!
IMPLICIT NONE
CHARACTER(LEN=2):: spieces
CHARACTER(LEN=4):: xyzcoord
CHARACTER(LEN=7):: xyzformat
CHARACTER(LEN=20):: temp
CHARACTER(LEN=64):: inputfile
CHARACTER(LEN=64):: comment
CHARACTER(LEN=128):: msg
INTEGER:: i, NP, nonfrac
REAL*8:: a0, testreal
REAL*8:: smass !Mass of atoms
REAL*8,DIMENSION(4):: P
!
!
!Initialize variables
a0 = 1.d0
P(:) = 0.d0
nonfrac = 0
xyzformat = 'massxyz'
xyzcoord = 'unkn'
 comment = 'File generated by pikashu'
!
!
100 CONTINUE
msg = 'debug --> entering READ_XYZ'
CALL DISPLAY_MSG(verbosity,msg,logfile)
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD')
OPEN(UNIT=35,FILE='pikashu.xyz',FORM='FORMATTED',STATUS='UNKNOWN')
!
!
140 CONTINUE
!In case of extended XYZ format, we have to read info at end of file
! and check if coordinates are cartesian or reduced coordinates
msg = 'debug --> determine if extended XYZ or not'
CALL DISPLAY_MSG(verbosity,msg,logfile)
DO
  READ(30,*,ERR=150,END=150) temp
  temp = TRIM(ADJUSTL(temp))
  IF(temp=='alat') THEN
    READ(30,*,ERR=500,END=500) a0
  ELSEIF(temp(1:5)=='super') THEN
    READ(30,*,ERR=500,END=500) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=500,END=500) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=500,END=500) H(3,1), H(3,2), H(3,3)
    Hset = .TRUE.
  ENDIF
ENDDO
!
!
150 CONTINUE
H(:,:) = a0*H(:,:)
msg = 'debug --> determine if format is massxyz or spiexyz'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!We have to determine if the format is
! (mass x y z)  or  (spieces x y z)
REWIND(30)
READ(30,*) temp   !NP
IF(temp=='0') GOTO 500
READ(30,*) temp   !Comment
READ(30,*,ERR=160,END=160) temp
READ(temp,*,ERR=160,END=500) testreal
!If it is a real, then the file has 'massxyz' format
xyzformat = 'massxyz'
GOTO 170
160 CONTINUE
!If it was not a real, then it has to be 'spiecesxyz' format
xyzformat = 'spiexyz'
!Let's confirm that: check that the string does correspond to
!a known spieces
READ(temp,*) spieces
CALL ATOMMASS(spieces,smass)
WRITE(msg,*) 'debug --> spiexyz: spieces, smass = ', spieces, smass
CALL DISPLAY_MSG(verbosity,msg,logfile)
IF(smass==0.d0) THEN
  i = -1
  GOTO 250
ENDIF
!
!
170 CONTINUE
msg = 'debug --> determine if coordinates are fractional or cartesian'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!We have to determine if the coordinates are cartesian or fractional
!For that we parse the file and compare the number of atoms with
!coordinates greater than 1, and the total number of atoms. If the
!ratio is very high then it is cartesian coordinates; if it is very low
!then it is fractional coordinates
IF(xyzcoord=='unkn') THEN
  REWIND(30)
  READ(30,*) NP
  READ(30,*) temp
  DO i=1,NP
    READ(30,*,ERR=250,END=250) temp, P(1), P(2), P(3)
    IF(P(1)>1.d0 .OR. P(2)>1.d0 .OR. P(3)>1.d0 ) nonfrac=nonfrac+1
  ENDDO
  testreal = REAL(nonfrac)/REAL(NP)
  IF(testreal>=0.7d0) THEN
    xyzcoord = 'cart'
    msg = '..> Found cartesian coordinates.'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  ELSEIF(testreal<=0.3d0) THEN
    xyzcoord = 'frac'
    msg = '..> Found fractional coordinates.'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  ELSE
    msg = '/!\ WARNING: I was unable to determine if the file '//inputfile
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    msg = '    contains cartesian or fractional coordinates.'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    nwarn = nwarn+1
  ENDIF
  WRITE(msg,*) 'debug --> threshold: ', testreal
  CALL DISPLAY_MSG(verbosity,msg,logfile)
ENDIF
!
!
!
200 CONTINUE
msg = 'debug --> coordinates are '//TRIM(xyzcoord)
CALL DISPLAY_MSG(verbosity,msg,logfile)
REWIND(30)
READ(30,*,ERR=210,END=500) NP
WRITE(35,*) NP
!
READ(30,'(a64)',ERR=210,END=500) comment
WRITE(35,*) TRIM(comment)
!
210 CONTINUE
!Read atomic positions
WRITE(msg,'(a36,i7)') 'debug --> read atomic positions; NP=', NP
CALL DISPLAY_MSG(verbosity,msg,logfile)
DO i=1,NP
  IF(xyzformat=='massxyz') THEN
    READ(30,*,ERR=250,END=250) P(4), P(1), P(2), P(3)
  ELSEIF(xyzformat=='spiexyz') THEN
    READ(30,*,ERR=250,END=250) temp, P(1), P(2), P(3)
    READ(temp,*) spieces
    CALL ATOMMASS(spieces,smass)
    P(4)=smass
  ENDIF
  !In case of fractional coordinates we have to convert them to cartesian
  IF(xyzcoord=='frac') THEN
    P(1) = P(1)*H(1,1)+P(1)*H(1,2)+P(1)*H(1,3)
    P(2) = P(2)*H(2,1)+P(2)*H(2,2)+P(2)*H(2,3)
    P(3) = P(3)*H(3,1)+P(3)*H(3,2)+P(3)*H(3,3)
  ENDIF
  WRITE(35,220) P(4), P(1), P(2), P(3)
ENDDO
220 FORMAT(f6.2,X,3(f10.6,2x))
GOTO 300
!
250 CONTINUE
msg = 'X!X ERROR: there was an error while parsing the file '//TRIM(inputfile)
CALL DISPLAY_MSG(verbosity,msg,logfile)
IF(i<=0) THEN
  WRITE(msg,*) '   Data appears to have an unrecognized format.'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
ELSE
  WRITE(msg,251) '   This apparently occured when reading atom #', i
  CALL DISPLAY_MSG(verbosity,msg,logfile)
ENDIF
251 FORMAT(a46,i7)
nerr = nerr+1
GOTO 500
!
!
300 CONTINUE
msg = 'debug --> write footer of pikashu.xyz'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!Write extended info at end of pikashu.xyz
WRITE(35,'(a4)') 'alat'
WRITE(35,'(a3)') '1.0'
WRITE(35,'(a9)') 'supercell'
WRITE(35,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(35,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(35,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
!
!
500 CONTINUE
CLOSE(30)
CLOSE(35)
!
END SUBROUTINE READ_XYZ
!
!
END MODULE in_xyz
