MODULE messages
!
!**********************************************************************************
!*  MESSAGES                                                                      *
!**********************************************************************************
!* This module deals with messages displayed by the PIKASHU program.              *
!**********************************************************************************
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 07 June 2010                                     *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!* List of subroutines in this module:                                            *
!* DISPLAY_MSG         displays a message depending on the verbosity level        *
!* DISPLAY_HEADER      displays the headline of the program                       *
!* DISPLAY_HELP        displays the help of the program                           *
!**********************************************************************************
!
!
USE comv
USE constants
!
IMPLICIT NONE
!
!
CONTAINS
!
!
!********************************************************
! DISPLAY_MSG
! This subroutine displays a message depending on the
! level of verbosity
!********************************************************
SUBROUTINE DISPLAY_MSG(verb,msg,logfile)
!
IMPLICIT NONE
CHARACTER(LEN=64):: logfile     !The log file to write messages into
CHARACTER(LEN=128):: msg, msg2  !The message itself
INTEGER:: verb   !Level of verbosity:
                 !0=silent, no message is ever displayed nor written in log file
                 !1=messages are written on the screen only
                 !2=messages are written in log file only
                 !3=messages are written in log+screen
                 !4=additional debugging messages (starting with 'debug')
                 !  are written in logfile
                 !The latter value (4) should be used only for
                 !programming/debugging purposes
!
msg2 = TRIM(ADJUSTL(msg))
IF( (verb>=2.AND.msg2(1:5).NE.'debug') .OR. verb==4 ) THEN
  OPEN(UNIT=20,FILE=logfile,FORM='FORMATTED',POSITION='APPEND')
  WRITE(20,*) TRIM(msg)
  CLOSE(20)
ENDIF
IF( (verb==1 .OR. verb>=3).AND.msg2(1:5).NE.'debug' ) THEN
  WRITE(*,*) TRIM(msg)
ENDIF
!
END SUBROUTINE DISPLAY_MSG
!
!
!********************************************************
! DISPLAY_HEADER
! This subroutine displays the "welcome message"
! of pikashu
!********************************************************
SUBROUTINE DISPLAY_HEADER()
!
IMPLICIT NONE
CHARACTER(LEN=128):: msg
!
!Print a nice message
msg = '************** P I K A S H U ****************'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = "PIerre's Swiss-army Knife for handling Atomic   \\___()"
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = "   file formats in a Simple and HUman way       (o^.^o)"
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '      ********'//version//'********            .   .    _'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '         This is free software, see:            /m   m\ _Z/'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '    http://www.gnu.org/licenses/gpl.html        | ( ) |Z7'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '        or refer to the LICENSE file.            @___@'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '              ***************'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
END SUBROUTINE DISPLAY_HEADER
!
!
!********************************************************
! DISPLAY_HELP
! This subroutine displays all or part of
! the help for pikashu
!********************************************************
SUBROUTINE DISPLAY_HELP(helpsection)
!
IMPLICIT NONE
CHARACTER(LEN=16):: helpsection
!
IF(TRIM(helpsection)=='') helpsection='general'
!
!
IF(helpsection=='general' .OR. helpsection=='') THEN
WRITE(*,*) '>>> USAGE:'
WRITE(*,*) '          pikashu [mode] <inputfile> <outputfile> [<formats>] [options]'
WRITE(*,*) '    where [] are optional parameters, and <> must be replaced'
WRITE(*,*) '    by specific values or strings.'
WRITE(*,*) ''
WRITE(*,*) '>>> FOR MORE HELP:'
WRITE(*,*) '..> On modes:'
WRITE(*,*) '          pikashu -help modes'
WRITE(*,*) ''
WRITE(*,*) '..> On options:'
WRITE(*,*) '          pikashu -help options'
WRITE(*,*) ''
ENDIF
!
IF(helpsection=='modes') THEN
WRITE(*,*) '>>> MODES:'
WRITE(*,*) '..> Interactive mode:'
WRITE(*,*) '          pikashu'
WRITE(*,*) ''
WRITE(*,*) '..> Normal mode:'
WRITE(*,*) '          pikashu <inputfile> [<outputfile>] [<formats>]'
WRITE(*,*) ''
WRITE(*,*) '..> List mode:'
WRITE(*,*) '          pikashu -l <listfile> [<formats>]'
WRITE(*,*) ''
WRITE(*,*) '..> Create mode:'
WRITE(*,*) '          pikashu -create <a0> <structure> <spieces> <outputfile>'
WRITE(*,*) ''
WRITE(*,*) '..> DDplot mode:'
WRITE(*,*) '          pikashu -dd <file1> <file2>'
WRITE(*,*) ''
WRITE(*,*) '..> Difference mode:'
WRITE(*,*) '          pikashu -diff <file1> <file2>'
WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options') THEN
  WRITE(*,*) '>>> OPTIONS:'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-expand' .OR. helpsection=='-e') THEN
  WRITE(*,*) '..> Expand the system in the 3 directions of space:'
  WRITE(*,*) '          -expand <ex> <ey> <ez>'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-deform' .OR. helpsection=='-def') THEN
  WRITE(*,*) '..> Apply uniaxial stress or strain:'
  WRITE(*,*) '          -deform <x/y/z> <strain> <Poissons ratio>'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-shift') THEN
  WRITE(*,*) '..> Shift part of the system:'
  WRITE(*,*) '          -shift <above/below> <distance> <x/y/z> <tau1> <tau2>'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-cut') THEN
  WRITE(*,*) '..> Cut part of the system:'
  WRITE(*,*) '          -cut <above/below> <cutdistance> <x/y/z>'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-rotate' .OR. helpsection=='-rot') THEN
  WRITE(*,*) '..> Rotate the system around an axis:'
  WRITE(*,*) '          -rotate <x/y/z> <angle_degrees>'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-dislocation' .OR. helpsection=='-disloc') THEN
  WRITE(*,*) '..> Insert a dislocation in the system:'
  WRITE(*,*) '          -disloc <edge/screw> <x/y/z> <b> <poisson> <pos1> <pos2>'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-alignx') THEN
  WRITE(*,*) '..> Align the first supercell vector with the X axis:'
  WRITE(*,*) '          -alignx'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-rebox') THEN
  WRITE(*,*) '..> (Re-)calculate vectors of the bounding box:'
  WRITE(*,*) '          -rebox'
  WRITE(*,*) ''
ENDIF
!
IF(helpsection=='options' .OR. helpsection=='-sort') THEN
  WRITE(*,*) '..> Sort atoms:'
  WRITE(*,*) '          -sort <s/x/y/z> <up/down>'
  WRITE(*,*) ''
ENDIF
!
WRITE(*,*) '>>> Look at the /doc folder of the program for more details.'
WRITE(*,*) ''
!
!
END SUBROUTINE DISPLAY_HELP
!
!
!
!
END MODULE messages
