MODULE mode_normal
!
!**********************************************************************************
!*  MODE_NORMAL                                                                   *
!**********************************************************************************
!* This module converts a single file to one or several other file(s).            *
!* It does that in three steps:                                                   *
!* 1- read the input file and convert it to extended XYZ                          *
!* 2- apply transformations to the system (deform, expand...)                     *
!* 3- read the extended XYZ and convert it to the format(s) asked by the user     *
!**********************************************************************************
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 25 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!* Structure of this module:                                                      *
!* 1-99   : define and initialize variables                                       *
!* 100-199: run a few checks on the files, determine file formats                 *
!* 200-299: read the input file, convert it to pikashu.xyz                        *
!* 300-399: determine system dimensions H (if not found in input file)            *
!* 400-499: apply options to the system (cut, expand, deform...)                  *
!* 500-599: convert pikashu.xyz to output files (cfg, xsf...)                     *
!* >800   : terminate program                                                     *
!**********************************************************************************
!
!Load modules
USE comv
USE constants
USE functions
USE messages
USE files
USE subroutines
USE guess_form
USE deterH
!
!Modules managing options
USE cut_cell
USE expand
USE deform
USE shift
USE rotate
USE alignx
USE dislocation
USE sort
! -- add other options here --
!
!Modules managing input files
USE readin
!
!Modules managing output files
USE out_gulp_gin
USE out_xyz
USE out_cfg
USE out_xsf
USE out_vasp_poscar
USE out_lammps_data
! -- add other formats here --
!
!
CONTAINS
!
SUBROUTINE CONVERT_AFF(inputfile,outputfile,outfileformat)
!
!
!
!Declare variables
IMPLICIT NONE
CHARACTER(LEN=3):: infileformat, outfileformat
CHARACTER(LEN=128):: temp
CHARACTER(LEN=128):: inputfile, outputfile
CHARACTER(LEN=128):: msg
INTEGER:: NP
INTEGER:: ioptions
INTEGER:: strlength
!
CHARACTER(LEN=10):: optionname
!Variables relative to Option: cut
CHARACTER(LEN=1):: cutdir        !x, y or z
CHARACTER(LEN=5):: cut_dir       !above or below
REAL*8:: cutdistance
!
!Variables relative to Option: expand
INTEGER, DIMENSION(3):: expandmatrix  !number of times the system must be
                                      !repeated in each direction of space
!
!Variables relative to Option: deform
CHARACTER(LEN=1):: def_dir       !direction of applied strain (X, Y or Z)
REAL*8:: def_strain, def_poisson  !applied strain and Poisson's ratio
!
!
!Variables relative to Option: shift
CHARACTER(LEN=1):: shift_axis     !(x, y or z)
CHARACTER(LEN=5):: shift_dir       !'above' or 'below'
REAL*8:: shift_dist       !distance of the "cut plane" to the origin
REAL*8:: shift_tau1, shift_tau2  !shift vector
!
!Variables relative to Option: rotate
CHARACTER(LEN=1):: rot_axis     !(x, y or z)
REAL*8:: rot_angle              !in degrees
!
!Variables relative to Option: dislocation
CHARACTER(LEN=1):: dislocline     !(x, y or z)
CHARACTER(LEN=5):: disloctype     !edge or screw
REAL*8:: b, nu, pos1, pos2
!
!Variables relative to Option: sort
CHARACTER(LEN=1):: sortcol
CHARACTER(LEN=4):: sortorder
! -- add other options global variables here --
!
!
!Initialize variables
NP = 0
H(:,:) = 0.d0
Hset=.FALSE.
infileformat=''
strlength=0
!
!If the interactive mode runs, then we must use the
!'outfileformat' defined previously
!Else, the output file formats are defined by the
!global logical variables (output_xyz, output_cfg, etc.)
!so we have to reset the 'outfileformat' variable
IF(mode.NE.'interactive') outfileformat = ''
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  READ INPUT FILE
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
100 CONTINUE
msg = 'debug --> CONVERT_AFF'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
temp = 'pikashu.xyz'
CALL READ_AFF(inputfile,temp)
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  DETERMINE H
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
300 CONTINUE
!!Determine if the cell vectors were found or not
IF(Hset) GOTO 400
IF(H(1,1)==0.d0 .AND. H(1,2)==0.d0 .AND. H(1,3)==0.d0) THEN
  Hset = .FALSE.
ELSE
  Hset = .TRUE.
ENDIF
!!If basis vectors H were not found in the input file, then compute them
! by calling the subroutine determine_H
! (this does not work perfectly yet, see determine_H.f90)
IF(.NOT.Hset) THEN
  msg = '>>> I did not find the cell vectors in the input file.'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  msg = '..> I will try to determine them...'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  !
  temp = 'pikashu.xyz'
  CALL DETERMINE_H(temp)
  !
  IF(nerr>=1) GOTO 800
  msg = '..> Cell vectors were computed (they may be wrong however).'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
ENDIF
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  APPLY OPTIONS TO THE SYSTEM
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
400 CONTINUE
msg = 'debug --> Base vectors:'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,410) 'debug -->      | ', H(1,1), H(1,2), H(1,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,410) 'debug -->  H = | ', H(2,1), H(2,2), H(2,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,410) 'debug -->      | ', H(3,1), H(3,2), H(3,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
410 FORMAT(a11,3(f10.5,2X),a1)
!
msg = 'debug --> APPLYING OPTIONS TO THE SYSTEM:'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!Options always read from and write to the file pikashu.xyz
temp='pikashu.xyz'
!
DO ioptions=1,20
  msg = 'debug --> option: '//TRIM(options_array(ioptions))
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  READ(options_array(ioptions),*,END=450,ERR=450) optionname
  optionname = TRIM(ADJUSTL(optionname))
  IF(optionname=='') EXIT
  !
  SELECT CASE(optionname)
  !
  CASE('-e', '-expand')
    READ(options_array(ioptions),*,END=450,ERR=450) optionname, &
        & expandmatrix(1), expandmatrix(2), expandmatrix(3)
    CALL EXPANDCELL(temp,temp,expandmatrix)
  !
  CASE('-def', '-deform')
    READ(options_array(ioptions),*,END=450,ERR=450) optionname, &
        & def_dir, def_strain, def_poisson
    CALL DEFORM_XYZ(temp,temp,def_dir,def_strain,def_poisson)
  !
  CASE('-shift')
    READ(options_array(ioptions),*,END=450,ERR=450) optionname, &
        & shift_dir, shift_dist, shift_axis, shift_tau1, shift_tau2
    CALL SHIFT_XYZ(temp,temp,shift_dir,shift_dist,shift_axis,shift_tau1,shift_tau2)
  !
  CASE('-cut')
    READ(options_array(ioptions),*,END=450,ERR=450) optionname, &
        & cut_dir, cutdistance, cutdir
    CALL CUTCELL(temp,temp,cut_dir,cutdistance,cutdir)
  !
  CASE('-rot', '-rotate')
    READ(options_array(ioptions),*,END=450,ERR=450) optionname, &
        & rot_axis, rot_angle
    CALL ROTATE_XYZ(temp,temp,rot_axis,rot_angle)
  !
  CASE('-disloc', '-dislocation')
    READ(options_array(ioptions),*,END=450,ERR=450) optionname, &
        & disloctype, dislocline, b, nu, pos1, pos2
    CALL DISLOC_XYZ(temp,temp,disloctype,dislocline,b,nu,pos1,pos2)
  !
  CASE('-alignx')
    CALL ALIGN_X(temp,temp)
  !
  CASE('-rebox')
    Hset = .FALSE.
    CALL DETERMINE_H(temp)
  !
  CASE('-sort')
    READ(options_array(ioptions),*,END=450,ERR=450) optionname, sortcol, sortorder
    CALL SORT_XYZ(temp,temp,sortcol,sortorder)
  !
  ! -- add other options here --
  !
  CASE DEFAULT
    nwarn = nwarn+1
    msg = '/!\ WARNING: could not understand this option: '//options_array(ioptions)
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    msg = '    Try `pikashu -help options` for a summary of options.'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  !
  END SELECT
!
ENDDO
!
450 CONTINUE
IF(nerr>=1) GOTO 800
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  WRITE OUTPUT FILE(S)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
500 CONTINUE
IF(outfileformat=='') CALL GUESS_FORMAT(outputfile,outfileformat)
!
msg = 'debug --> outfileformat: '//outfileformat
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
520 CONTINUE
SELECT CASE(outfileformat)
CASE('xyz')
  output_xyz = .TRUE.
CASE('cfg')
  output_cfg = .TRUE.
CASE('xsf')
  output_xsf = .TRUE.
CASE('gin')
  output_gin = .TRUE.
CASE('pos')
  output_poscar = .TRUE.
! -- add other formats here --
!
CASE DEFAULT
  CONTINUE
END SELECT
!
!If no output format is active then we have to ask the user
IF( .NOT.output_xyz    .AND.  .NOT.output_cfg .AND.            &
  & .NOT.output_xsf    .AND.  .NOT.output_gin .AND.            &
  & .NOT.output_poscar                                         &
  ! -- add other formats here --
  &                                                 ) THEN
    nwarn = nwarn+1
    msg = '/!\ WARNING: I could not guess the format for the output file: '//TRIM(outputfile)
    CALL DISPLAY_MSG(3,msg,logfile)
    msg = '(°< Please specify a format for the output file:'
    CALL DISPLAY_MSG(3,msg,logfile)
    msg = '    '//listofformats
    CALL DISPLAY_MSG(3,msg,logfile)
    READ(*,*) outfileformat
    CALL NAME_OUTFILE(outputfile,outputfile,outfileformat)
    GOTO 520
ENDIF
!
msg = '>>> Writing output file(s):'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
!!Then, write output file
! by calling the module corresponding to the output format
! Output to several formats is possible
!
IF(output_xyz) THEN
  CALL NAME_OUTFILE(outputfile,outputfile,'xyz')
  IF(.NOT.overw) CALL CHECKFILE(outputfile,'writ')
  CALL WRITE_XYZ(outputfile)
ENDIF
IF(nerr>=1) GOTO 800
!
IF(output_cfg) THEN
  CALL NAME_OUTFILE(outputfile,outputfile,'cfg')
  IF(.NOT.overw) CALL CHECKFILE(outputfile,'writ')
  CALL WRITE_CFG(outputfile)
ENDIF
IF(nerr>=1) GOTO 800
!
IF(output_xsf) THEN
  CALL NAME_OUTFILE(outputfile,outputfile,'xsf')
  IF(.NOT.overw) CALL CHECKFILE(outputfile,'writ')
  CALL WRITE_XSF(outputfile)
ENDIF
IF(nerr>=1) GOTO 800
!
IF(output_gin) THEN
  CALL NAME_OUTFILE(outputfile,outputfile,'gin')
  IF(.NOT.overw) CALL CHECKFILE(outputfile,'writ')
  CALL WRITE_GIN(outputfile)
ENDIF
IF(nerr>=1) GOTO 800
!
IF(output_lammps) THEN
  CALL NAME_OUTFILE(outputfile,outputfile,'lmp')
  IF(.NOT.overw) CALL CHECKFILE(outputfile,'writ')
  CALL WRITE_LMP_DATA(outputfile)
ENDIF
IF(nerr>=1) GOTO 800
!
! -- add other formats here --
!
!POSCAR is a bit special because the file name is 'POSCAR'
!instead of being built from the 'outputfile' + an extension
IF(output_poscar) THEN
  temp = 'POSCAR'
  IF(.NOT.overw) CALL CHECKFILE(temp,'writ')
  CALL WRITE_POSCAR(temp)
ENDIF
IF(nerr>=1) GOTO 800
!
GOTO 900
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  ERROR MESSAGES
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
800 CONTINUE
msg = 'X!X ERROR: there were errors while processing the files.'
CALL DISPLAY_MSG(3,msg,logfile)
msg = '          File '//TRIM(ADJUSTL(inputfile))//' was not fully converted.'
GOTO 1000
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  TERMINATE MODULE
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
900 CONTINUE
IF(nerr>=1) GOTO 1000
!Terminating program
msg = '>>> File '//TRIM(ADJUSTL(inputfile))//' was converted.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
1000 CONTINUE
IF(verbosity<4) CALL SYSTEM('rm -f pikashu.xyz')
!
END SUBROUTINE CONVERT_AFF
!
!
END MODULE mode_normal
