MODULE alignx
!
!**********************************************************************************
!*  ALIGNXY                                                                       *
!**********************************************************************************
!* This module reads a file in the extended XYZ format and                        *
!* rotates the system so the first axis is aligned with X, i.e. so                *
!* the H matrix is transformed into a lower triangular matrix H':                 *
!*                     |  H'(1,1)     0        0     |                            *
!*                H' = |  H'(2,1)  H'(2,2)     0     |                            *
!*                     |  H'(3,1)  H'(3,2)  H'(3,3)  |                            *
!* Atomic positions are also transformed accordingly:                             *
!*                R' = H^-1 R H'                                                  *
!* Then it outputs the result in a XYZ file.                                      *
!**********************************************************************************
!* (C) May 2010 - Pierre Hirel                                                    *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 17 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE functions
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
!
SUBROUTINE ALIGN_X(inputfile,outputfile)
!
!
IMPLICIT NONE
CHARACTER(LEN=128):: file, inputfile, outputfile
CHARACTER(LEN=128):: temp, msg, comment
INTEGER:: i, NP
REAL*8:: a0
REAL*8:: alpha, beta, gamma
REAL*8:: Hx, Hy, Hz !Length of H(1,:), H(2,:) and H(3,:)
REAL*8:: P1, P2, P3
REAL*8,DIMENSION(:,:),ALLOCATABLE:: P
REAL*8,DIMENSION(3,3):: G
!
!
!Initialize variables
i = 0
a0 = 1.d0
P1 = 0.d0
P2 = 0.d0
P3 = 0.d0
alpha = 0.d0
beta  = 0.d0
gamma = 0.d0
Hx = 0.d0
Hy = 0.d0
Hz = 0.d0
G(:,:) = 0.d0
!
msg = 'debug --> Entering ALIGN_X'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
msg = '>>> Aligning 1st base vector with X axis...'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
IF( H(1,2)==0.d0 .AND. H(1,3)==0.d0 ) THEN
  msg = '..> Axis is already aligned, skipping.'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  GOTO 1000
ENDIF
!
!
!
100 CONTINUE
!Read input file
file = inputfile
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!Read extended info at end of file
DO
  READ(30,*,ERR=110,END=110) temp
  IF(temp(1:4)=='alat') THEN
    READ(30,*,ERR=800,END=800) a0
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(30,*,ERR=800,END=800) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=800,END=800) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=800,END=800) H(3,1), H(3,2), H(3,3)
    msg = 'debug --> Found supercell vectors'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    Hset=.TRUE.
  ENDIF
  !IF(temp(1:4)=='Redu'.OR.temp(1:4)=='redu') reduced=.TRUE.
  !IF(temp(1:4)=='Cart'.OR.temp(1:4)=='cart') reduced=.FALSE.
ENDDO
!
110 CONTINUE
H(:,:) = a0*H(:,:)
!
!Calculate the inverse matrix G=H^-1 for later conversion of coordinates
CALL INVMAT(H,G)
!
!Now lets convert H into conventional notation
Hx=DSQRT( H(1,1)**2.d0+H(1,2)**2.d0+H(1,3)**2.d0 )
Hy=DSQRT( H(2,1)**2.d0+H(2,2)**2.d0+H(2,3)**2.d0 )
Hz=DSQRT( H(3,1)**2.d0+H(3,2)**2.d0+H(3,3)**2.d0 )
alpha = ANGVEC(H(2,:),H(3,:))
beta  = ANGVEC(H(3,:),H(1,:))
gamma = ANGVEC(H(1,:),H(2,:))
!
!And then convert this conventional notation into Cartesian matrix H'
CALL CONVMAT(Hx,Hy,Hz,alpha,beta,gamma,H)
!
!
!
200 CONTINUE
!Now read atomic positions and convert them to fractional coord.
REWIND(30)
READ(30,*) NP
READ(30,'(a128)') comment
!
ALLOCATE(P(4,NP))
P(:,:) = 0.d0
DO i=1,NP
  READ(30,*,ERR=800,END=800) P(4,i), P1, P2, P3
  P(1,i) = P1*G(1,1) + P2*G(1,2) + P3*G(1,3)
  P(2,i) = P1*G(2,1) + P2*G(2,2) + P3*G(2,3)
  P(3,i) = P1*G(3,1) + P2*G(3,2) + P3*G(3,3)
ENDDO
!
CLOSE(30)
!
!
!
300 CONTINUE
!Finally, write atomic positions
OPEN(UNIT=31,FILE=outputfile,FORM='FORMATTED',STATUS='UNKNOWN')
WRITE(31,*) NP
WRITE(31,*) TRIM(comment)
!
DO i=1,NP
  WRITE(31,310) P(4,i), P(1,i)*H(1,1)+P(2,i)*H(2,1)+P(3,i)*H(3,1), &
             &          P(1,i)*H(1,2)+P(2,i)*H(2,2)+P(3,i)*H(3,2), &
             &          P(1,i)*H(1,3)+P(2,i)*H(2,3)+P(3,i)*H(3,3)
ENDDO
310 FORMAT(f6.2,X,3(f10.6,2x))
!
!Write extended info at the end of XYZ file
WRITE(31,'(a4)') 'alat'
WRITE(31,'(a3)') '1.0'
WRITE(31,'(a9)') 'supercell'
WRITE(31,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(31,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(31,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
!
!
!
400 CONTINUE
CLOSE(31)
msg = '..> Axis was successfully aligned.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading '//TRIM(file)
CALL DISPLAY_MSG(3,msg,logfile)
WRITE(msg,*) '    This apparently occured while trying to read atom ', i
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
1000 CONTINUE
IF(ALLOCATED(P)) DEALLOCATE(P)
!
!
!
END SUBROUTINE ALIGN_X
!
!
END MODULE alignx
