MODULE cut_cell
!
!**********************************************************************************
!*  CUT_CELL                                                                      *
!**********************************************************************************
!* This module reads a file in the extended XYZ format and "cuts" it.             *
!* This is useful if one wants to reduce the number of atoms displayed.           *
!**********************************************************************************
!* (C) March 2010 - Pierre Hirel                                                  *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 11 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
!
SUBROUTINE CUTCELL(inputfile,outputfile,cut_dir,cutdistance,cutdir)
!
IMPLICIT NONE
CHARACTER(LEN=1):: cutdir         !x, y or z
CHARACTER(LEN=5):: cut_dir       !above or below
CHARACTER(LEN=128):: inputfile, outputfile
CHARACTER(LEN=128):: comment, msg
INTEGER:: a1, a2, a3
INTEGER:: i, NP, NPcut
REAL*8:: a0
REAL*8:: cutdistance
REAL*8,DIMENSION(:,:),ALLOCATABLE:: P
!
!
IF(cutdistance==0) GOTO 1000
!
!Initialize variables
i = 0
NP = 0
NPcut = 0
!
!
WRITE(msg,5) '>>> Cutting the system ', cut_dir, cutdistance,' A along ', cutdir
CALL DISPLAY_MSG(verbosity,msg,logfile)
5 FORMAT(a23,a5,X,f7.2,a9,a1)
!
!
!Define the axes
IF(cutdir=='x' .OR. cutdir=='X') THEN
  a1 = 1
  a2 = 2
  a3 = 3
ELSEIF(cutdir=='y' .OR. cutdir=='Y') THEN
  a1 = 2
  a2 = 3
  a3 = 1
ELSEIF(cutdir=='z' .OR. cutdir=='Z') THEN
  a1 = 3
  a2 = 1
  a3 = 2
ELSE
  nerr = nerr+1
  msg = 'X!X ERROR: I could not determine cut direction: '//TRIM(cutdir)
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  GOTO 1000
ENDIF
!
!
100 CONTINUE
OPEN(UNIT=31,FILE=inputfile,FORM='FORMATTED',STATUS='OLD')
!Read inputfile and count number of atoms in the "cutted" slab
READ(31,*) NP
READ(31,'(a128)') comment
ALLOCATE(P(NP,4))
P(:,:)=0.d0
NPcut = 0
!
WRITE(msg,*) 'debug --> NP: ', NP
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
DO i=1,NP
  READ(31,*,ERR=800,END=800) P(i,4), P(i,1), P(i,2), P(i,3)
  IF( cut_dir=='above' .AND. P(i,a1)>cutdistance .OR.        &
    & cut_dir=='below' .AND. P(i,a1)<cutdistance      ) THEN
    !If atom is out of the region of interest, we mark it by
    !setting P(4) (which is supposed to refer to the atom mass)
    !to an arbitrary negative value
    P(i,4) = -1.d0
  ELSE
    !Else, we count this atom as part of the region of interest
    NPcut = NPcut+1
  ENDIF
ENDDO
!
WRITE(msg,*) 'debug --> NPcut:', NPcut
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,'(a19,i8)') '..> Atoms removed: ', NP-NPcut
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
!
200 CONTINUE
CLOSE(31)
!Write atomic positions of the "cutted" slab in the output file
OPEN(UNIT=32,FILE=outputfile,FORM='FORMATTED',STATUS='UNKNOWN')
WRITE(32,*) NPcut
WRITE(32,*) TRIM(comment)
!
DO i=1,NP
  IF(P(i,4)>0.d0) THEN
    WRITE(32,*) P(i,4), P(i,1), P(i,2), P(i,3)
  ENDIF
ENDDO
!
WRITE(32,'(a4)') 'alat'
WRITE(32,'(a3)') '1.0'
WRITE(32,'(a9)') 'supercell'
WRITE(32,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(32,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(32,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
!
CLOSE(32)
!
msg = '..> System was cut.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading '//TRIM(inputfile)
CALL DISPLAY_MSG(3,msg,logfile)
WRITE(msg,*) '    This apparently occured while trying to read atom ', i
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
1000 CONTINUE
IF(ALLOCATED(P)) DEALLOCATE(P)
!
!
END SUBROUTINE CUTCELL
!
END MODULE cut_cell
