MODULE deform
!
!**********************************************************************************
!*  DEFORM                                                                        *
!**********************************************************************************
!* This module reads a file in the extended XYZ format and                        *
!* applies uniaxial stress according to specified strain and Poisson's ratio.     *
!* Then it outputs the result in a XYZ file.                                      *
!**********************************************************************************
!* (C) March 2010 - Pierre Hirel                                                  *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 28 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
!
SUBROUTINE DEFORM_XYZ(inputfile,outputfile,def_dir,def_strain,def_poisson)
!
!
IMPLICIT NONE
CHARACTER(LEN=1):: def_dir  !direction of applied strain (X, Y or Z)
CHARACTER(LEN=128):: file, inputfile, outputfile
CHARACTER(LEN=128):: temp, msg, comment
INTEGER:: a1, a2, a3
INTEGER:: i, NP
REAL*8:: a0
REAL*8:: def_strain, def_poisson  !applied strain and Poisson's ratio
REAL*8:: P1, P2, P3
REAL*8,DIMENSION(4):: P
REAL*8,DIMENSION(3,3):: meps
!
!
!Initialize variables
i = 0
a0 = 1.d0        !If lattice parameter is not specified, default has to be 1
a1 = 0
a2 = 0
a3 = 0
P(:) = 0.d0
meps(:,:) = 0.d0
!
!
WRITE(msg,5) '>>> Deforming the system by ', def_strain,'% along ', def_dir
CALL DISPLAY_MSG(verbosity,msg,logfile)
5 FORMAT(a28,f8.4,a8,a1)
IF(def_poisson==0.d0) THEN
  msg = '..> Uniaxial strain.'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
ELSE
  WRITE(msg,'(a36,f8.4)') '..> Uniaxial stress, Poisson ratio: ', def_poisson
  CALL DISPLAY_MSG(verbosity,msg,logfile)
ENDIF
!
!
!Define the axes
IF(def_dir=='x' .OR. def_dir=='X') THEN
  a1 = 1
  a2 = 2
  a3 = 3
ELSEIF(def_dir=='y' .OR. def_dir=='Y') THEN
  a1 = 2
  a2 = 3
  a3 = 1
ELSEIF(def_dir=='z' .OR. def_dir=='Z') THEN
  a1 = 3
  a2 = 1
  a3 = 2
ELSE
  nerr = nerr+1
  msg = 'X!X ERROR: I could not determine deformation direction: '//TRIM(def_dir)
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  GOTO 1000
ENDIF
!
!Calculate the deformation matrix
!def_strain, def_poisson
meps(:,:) = 0.d0
meps(a1,a1) = 1.d0+def_strain/100.d0
meps(a2,a2) = 1.d0-def_poisson*def_strain/100.d0
meps(a3,a3) = 1.d0-def_poisson*def_strain/100.d0
!
!
!
100 CONTINUE
file = inputfile
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!Read extended info at end of file
DO
  READ(30,*,ERR=110,END=110) temp
  IF(temp(1:4)=='alat') THEN
    READ(30,*,ERR=800,END=800) a0
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(30,*,ERR=800,END=800) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=800,END=800) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=800,END=800) H(3,1), H(3,2), H(3,3)
    msg = 'debug --> Found supercell vectors'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    Hset=.TRUE.
  ENDIF
  !IF(temp(1:4)=='Redu'.OR.temp(1:4)=='redu') reduced=.TRUE.
  !IF(temp(1:4)=='Cart'.OR.temp(1:4)=='cart') reduced=.FALSE.
ENDDO
!
110 CONTINUE
H(:,:) = a0*H(:,:)
!
!Apply the strain to the box
H(1,1) = H(1,1)*meps(1,1)
H(2,2) = H(2,2)*meps(2,2)
H(3,3) = H(3,3)*meps(3,3)
!
msg = 'debug --> New base vectors:'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,150) 'debug -->      | ', H(1,1), H(1,2), H(1,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,150) 'debug -->  H = | ', H(2,1), H(2,2), H(2,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,150) 'debug -->      | ', H(3,1), H(3,2), H(3,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
150 FORMAT(a17,3(f10.5,2X),a1)
!
!
!
200 CONTINUE
REWIND(30)
OPEN(UNIT=31,FILE='pikashu_deform.xyz',FORM='FORMATTED',STATUS='UNKNOWN')
!Read header
REWIND(30)
READ(30,*,ERR=800,END=800) NP
WRITE(31,*) NP
READ(30,'(a128)',ERR=800,END=800) comment
WRITE(31,*) TRIM(comment)
!
!Read positions, apply strain and write in 'pikashu_deform.xyz'
P(:) = 0.d0
DO i=1,NP
  READ(30,*,ERR=800,END=800) P(4), P1, P2, P3
  P(1) = P1*meps(1,1)
  P(2) = P2*meps(2,2)
  P(3) = P3*meps(3,3)
  WRITE(31,210) P(4), P(1), P(2), P(3)
ENDDO
210 FORMAT(f6.2,X,3(f10.6,2x))
!
!Write extended info at the end of file
WRITE(31,'(a4)') 'alat'
WRITE(31,'(a3)') '1.0'
WRITE(31,'(a9)') 'supercell'
WRITE(31,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(31,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(31,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
!
!
!
300 CONTINUE
CLOSE(30)
CLOSE(31)
!Create the outputfile
CALL SYSTEM('cp -rf pikashu_deform.xyz '//outputfile)
msg = '..> System was deformed.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading '//TRIM(file)
CALL DISPLAY_MSG(3,msg,logfile)
WRITE(msg,*) '    This apparently occured while trying to read atom ', i
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
1000 CONTINUE
IF(verbosity<4) CALL SYSTEM('rm -f pikashu_deform.xyz')
!
!
END SUBROUTINE DEFORM_XYZ
!
!
END MODULE deform
