MODULE dislocation
!
!**********************************************************************************
!*  DISLOCATION                                                                   *
!**********************************************************************************
!* This module reads a file in the extended XYZ format and                        *
!* introduces a straight edge or screw dislocation.                               *
!* Then it outputs the result in a XYZ file.                                      *
!**********************************************************************************
!* (C) May 2010 - Pierre Hirel                                                    *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 28 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
!
SUBROUTINE DISLOC_XYZ(inputfile,outputfile,disloctype,dislocline,b,nu,pos1,pos2)
!
!
IMPLICIT NONE
CHARACTER(LEN=1):: dislocline
CHARACTER(LEN=5):: disloctype
CHARACTER(LEN=64):: file, inputfile, outputfile
CHARACTER(LEN=128):: temp, msg, comment
INTEGER:: a1, a2, a3
INTEGER:: i, k, NP
REAL*8:: a0
REAL*8:: b    !Length of Burgers vector (in Angstroms)
REAL*8:: nu   !Poissons ratio of the material
REAL*8:: pos1, pos2 !Coordinates of dislocation in the plane
REAL*8,DIMENSION(:,:),ALLOCATABLE:: P
!
!Initialize variables
a0 = 1.d0
 comment = 'File generated by pikashu'
!
msg = 'debug --> Entering DISLOC_XYZ'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
WRITE(msg,5) '>>> Inserting a ', TRIM(disloctype), &
     &       ' dislocation with line along ', dislocline, ','
CALL DISPLAY_MSG(verbosity,msg,logfile)
5 FORMAT( a16,a5,a29,a1,a1)
WRITE(msg,6) '    b=', b, ' A at (', pos1, ';', pos2, ')'
CALL DISPLAY_MSG(verbosity,msg,logfile)
6 FORMAT( a6,f8.3,a7,2(f8.3,a1) )
!
!Check that the values make sense
IF(b==0.d0) GOTO 1000
IF(pos1<0.d0) pos1 = DABS(pos1)
IF(pos2<0.d0) pos2 = DABS(pos2)
!
!Define the axes: a3=dislocation line
IF(dislocline=='x' .OR. dislocline=='X') THEN
  a1 = 2
  a2 = 3
  a3 = 1
ELSEIF(dislocline=='y' .OR. dislocline=='Y') THEN
  a1 = 3
  a2 = 1
  a3 = 2
ELSEIF(dislocline=='z' .OR. dislocline=='Z') THEN
  a1 = 1
  a2 = 2
  a3 = 3
ELSE
  msg = 'X!X ERROR: could not recognize axis: '//dislocline
  CALL DISPLAY_MSG(3,msg,logfile)
  nerr = nerr+1
  GOTO 1000
ENDIF
!
!If system is too small in the plane normal to the disloc.line,
!we output a warning message
!IF(H(
!
!
!
100 CONTINUE
file = inputfile
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!Read extended info at end of file
DO
  READ(30,*,ERR=110,END=110) temp
  IF(temp(1:4)=='alat') THEN
    READ(30,*,ERR=800,END=800) a0
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(30,*,ERR=800,END=800) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=800,END=800) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=800,END=800) H(3,1), H(3,2), H(3,3)
    msg = 'debug --> Found supercell vectors'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    Hset=.TRUE.
  ENDIF
ENDDO
!
110 CONTINUE
!If pos1 and pos2 are fractional, convert them to cartesian
IF(pos1<=1.d0 .AND. pos2<=1.d0) THEN
  pos1 = pos1*H(a1,a1)
  pos2 = pos2*H(a2,a2)
ENDIF
!
120 CONTINUE
!Read atomic positions from inputfile
REWIND(30)
READ(30,*,ERR=800,END=800) NP
READ(30,'(a128)',ERR=800,END=800) comment
!
!Allocate array for atomic positions
!Note that for edge dislocations we will introduce new atoms
! so we need a larger array
IF(disloctype=='screw') THEN
  ALLOCATE(P(NP,4))
ELSEIF(disloctype=='edge') THEN
  ALLOCATE(P(2*NP,4))
ENDIF
P(:,:) = 0.d0
!
DO i=1,NP
  READ(30,*,ERR=800,END=800) P(i,4), P(i,1), P(i,2), P(i,3)
ENDDO
CLOSE(30)
!
!
!
200 CONTINUE
!Apply atomic displacements
!(subroutines DISPEDGE and DISPSCREW are at the end of this module)
!
! SCREW DISLOCATION
IF(disloctype=='screw') THEN
  DO i=1,NP
    CALL DISPSCREW(P(i,:),a1,a2,a3,b,pos1,pos2)
  ENDDO
!
!
! EDGE DISLOCATION
ELSEIF(disloctype=='edge') THEN
  !First we insert a plane of atoms by copying
  !an existing plane
  k = 0
  DO i=1,NP
    IF( P(i,a1)>=pos1+DABS(b)            .AND.                  &
      & P(i,a1)<pos1+2.d0*DABS(b)-0.01d0 .AND.                  &
      & (P(i,a2)-pos2)*b/DABS(b)>=0.d0            ) THEN
      k=k+1
      P(NP+k,1) = P(i,1)
      P(NP+k,2) = P(i,2)
      P(NP+k,3) = P(i,3)
      P(NP+k,4) = P(i,4)
      P(NP+k,a1) = P(i,a1) -DABS(b)*1.25d0
      P(NP+k,a2) = P(i,a2) -0.1d0
    ENDIF
  ENDDO
  WRITE(msg,'(a30,i7)') '..> Number of atoms inserted: ', k
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  !
  !Then we apply atomic displacements to all atoms
  !(except the ones we just inserted)
  DO i=1,NP
    IF( P(i,a1)>=pos1 ) THEN
      P(i,a1) = P(i,a1)+DABS(b)/2.d0
    ENDIF
    CALL DISPEDGE(P(i,:),a1,a2,b,nu,pos1,pos2)
  ENDDO
  !
  !From now on, work with all atoms
  NP = NP+k
  !
  !Supercell has been expanded along a1: calculate the new length
  H(a1,a1) = H(a1,a1)+DABS(b)/1.6d0
!
ENDIF
!
!
!
300 CONTINUE
!Write positions into the outputfile
OPEN(UNIT=31,FILE=outputfile,FORM='FORMATTED',STATUS='UNKNOWN',POSITION='REWIND')
WRITE(31,*) NP
WRITE(31,*) TRIM(comment)
DO i=1,NP
  WRITE(31,310) P(i,4), P(i,1), P(i,2), P(i,3)
ENDDO
310 FORMAT(f6.2,X,3(f10.6,2x))
!Write extended info at the end of file
WRITE(31,'(a4)') 'alat'
WRITE(31,'(a3)') '1.0'
WRITE(31,'(a9)') 'supercell'
WRITE(31,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(31,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(31,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
CLOSE(31)
!
!
!
400 CONTINUE
msg = '..> Dislocation was inserted.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading '//TRIM(file)
CALL DISPLAY_MSG(3,msg,logfile)
WRITE(msg,*) '    This apparently occured while trying to read atom ', i
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
1000 CONTINUE
IF(ALLOCATED(P)) DEALLOCATE(P)
IF(verbosity<4) CALL SYSTEM('rm -f pikashu_disloc.xyz')
!
!
END SUBROUTINE DISLOC_XYZ
!
!
!
!********************************************************
! DISPEDGE
! This subroutine calculates the edge displacements.
! Formulae can be found for instance in
! J.P. Hirth, J. Lothe, 'Theory of dislocations',
! 1st ed. (1968), p.75
!********************************************************
!
SUBROUTINE DISPEDGE(P,a1,a2,b,nu,pos1,pos2)
!
IMPLICIT NONE
INTEGER,INTENT(IN):: a1, a2
REAL*8,INTENT(IN):: b          !Burgers vector
REAL*8,INTENT(IN):: nu
REAL*8,INTENT(IN):: pos1, pos2 !Position of the dislocation in the plane
                               !orthogonal to the dislocation line
REAL*8,DIMENSION(3),INTENT(INOUT):: P  !Atomic positions
!
P(a1) = P(a1) + b/(2.d0*pi)*                                &
          &  (ATAN( (P(a2)-pos2)/(P(a1)-pos1) ) +           &
          &   (P(a1)-pos1)*(P(a2)-pos2)/( 2.d0*(1.d0-nu)*   &
          &   ( (P(a1)-pos1)**2.d0+(P(a2)-pos2)**2.d0) )    &
          &  )
P(a2) = P(a2) - b/(2.d0*pi)*                                &
          &  ( (1.d0-2.d0*nu)/(4.d0*(1.d0-nu))*             &
          &   DLOG((P(a1)-pos1)**2.d0+(P(a2)-pos2)**2.d0) + &
          &   ( (P(a1)-pos1)**2.d0-(P(a2)-pos2)**2.d0 )/    &
          &   ( 4.d0*(1.d0-nu)*                             &
          &     ( (P(a1)-pos1)**2.d0+(P(a2)-pos2)**2.d0 )   &
          &   )                                             &
          &  )
!
END SUBROUTINE
!
!
!********************************************************
! DISPSCREW
! This subroutine calculates the edge displacements.
! Formulae can be found for instance in
! J.P. Hirth, J. Lothe, 'Theory of dislocations',
! 1st ed. (1968), p.59; or in
! D.Hull,D.J.Bacon,'Introduction to dislocations',
! 4th Ed.(2001),p.66.
!********************************************************
!
SUBROUTINE DISPSCREW(P,a1,a2,a3,b,pos1,pos2)
!
IMPLICIT NONE
INTEGER,INTENT(IN):: a1, a2, a3
REAL*8,INTENT(IN):: b           !Burgers vector
REAL*8,INTENT(IN):: pos1, pos2  !Position of the dislocation in the plane
                                !orthogonal to the dislocation line
REAL*8,DIMENSION(3),INTENT(INOUT):: P  !Atomic positions
!
P(a3) = P(a3) + b/(2.d0*pi)*                                &
     &  DATAN2( (P(a2)-pos2) , (P(a1)-pos1) )
!
END SUBROUTINE DISPSCREW
!
!
END MODULE dislocation