MODULE expand
!
!**********************************************************************************
!*  EXPAND                                                                        *
!**********************************************************************************
!* This module reads a file in the extended XYZ format, and                       *
!* expands it along the directions of the cell vectors. Then it outputs           *
!* a file in the extended XYZ format.                                             *
!**********************************************************************************
!* (C) March 2010 - Pierre Hirel                                                  *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 10 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
CONTAINS
!
!
SUBROUTINE EXPANDCELL(inputfile,outputfile,expandmatrix)
!
!
IMPLICIT NONE
CHARACTER(LEN=128):: file, inputfile, outputfile
CHARACTER(LEN=128):: temp, msg, comment
LOGICAL:: reduced
INTEGER:: i, NP, newNP
INTEGER:: m, n, o
INTEGER, DIMENSION(3):: expandmatrix
REAL*8:: a0
REAL*8,DIMENSION(:,:),ALLOCATABLE:: P
!
!Initialize variables
i = 0
!
!If one or more dimensions are set to 0 then replace it by 1
!(a zero dimension would delete all the atoms)
IF(expandmatrix(1)==0) expandmatrix(1)=1
IF(expandmatrix(2)==0) expandmatrix(2)=1
IF(expandmatrix(3)==0) expandmatrix(3)=1
!
WRITE(msg,'(a26,3i5)') '>>> Expanding the system: ', (expandmatrix(i), i=1,3)
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!If all dimensions are set to 1 then the system stays the same
IF( expandmatrix(1)==1 .AND. expandmatrix(2)==1 .AND.                 &
  & expandmatrix(3)==1                                ) GOTO 1000
!
!Initialize variables
a0 = 1.d0        !If lattice parameter is not specified, default has to be 1
reduced=.FALSE.  !By default, XYZ format uses cartesian coordinates
file = inputfile
!
!
100 CONTINUE
!Open the input file
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!
!Read number of particles and comment
READ(30,*) NP
READ(30,'(a128)') comment
ALLOCATE(P(4,NP))
P(:,:) = 0.d0
!
!Read atomic positions
DO i=1,NP
  READ(30,*,ERR=800,END=800) P(4,i), P(1,i), P(2,i), P(3,i)
ENDDO
!
!Read extended info at end of file
DO
  READ(30,*,ERR=120,END=120) temp
  IF(temp(1:4)=='alat') THEN
    READ(30,*,ERR=800,END=800) a0
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(30,*,ERR=800,END=800) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=800,END=800) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=800,END=800) H(3,1), H(3,2), H(3,3)
    msg = 'debug --> Found supercell vectors'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    Hset=.TRUE.
  ENDIF
  IF(temp(1:4)=='Redu'.OR.temp(1:4)=='redu') reduced=.TRUE.
  IF(temp(1:4)=='Cart'.OR.temp(1:4)=='cart') reduced=.FALSE.
ENDDO
!
120 CONTINUE
CLOSE(30)
!
H(:,:) = a0*H(:,:)
!
!
!
200 CONTINUE
!Expand the cell and write it in output file
OPEN(UNIT=32,FILE=outputfile,FORM='FORMATTED',STATUS='UNKNOWN',POSITION='REWIND')
!
newNP = NP*expandmatrix(1)*expandmatrix(2)*expandmatrix(3)
WRITE(msg,201) '..> New number of particles: ', newNP
CALL DISPLAY_MSG(verbosity,msg,logfile)
201 FORMAT(a29,i8)
!
WRITE(32,*) newNP
WRITE(32,*) TRIM(comment)
!
DO i=1,NP
  DO o=1,expandmatrix(3)
    DO n=1,expandmatrix(2)
      DO m=1,expandmatrix(1)
        WRITE(32,210) P(4,i),                                              &
             & P(1,i)+REAL(m-1)*H(1,1)+REAL(n-1)*H(2,1)+REAL(o-1)*H(3,1),  &
             & P(2,i)+REAL(m-1)*H(1,2)+REAL(n-1)*H(2,2)+REAL(o-1)*H(3,2),  &
             & P(3,i)+REAL(m-1)*H(1,3)+REAL(n-1)*H(2,3)+REAL(o-1)*H(3,3)
      ENDDO
    ENDDO
  ENDDO
ENDDO
210 FORMAT(f6.2,X,3(f10.6,2x))
!
!
!Resize the cell dimensions
H(1,:) = expandmatrix(1)*H(1,:)
H(2,:) = expandmatrix(2)*H(2,:)
H(3,:) = expandmatrix(3)*H(3,:)
!
!Write footer of XYZ file
WRITE(32,'(a4)') 'alat'
WRITE(32,'(a3)') '1.0'
WRITE(32,'(a9)') 'supercell'
WRITE(32,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(32,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(32,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
!
!
!
500 CONTINUE
CLOSE(32)
msg = '..> System was expanded.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading '//TRIM(file)
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
1000 CONTINUE
IF(ALLOCATED(P)) DEALLOCATE(P)
!
!
!
END SUBROUTINE EXPANDCELL
!
!
END MODULE expand