MODULE rotate
!
!**********************************************************************************
!*  ROTATE                                                                        *
!**********************************************************************************
!* This module reads a file in the extended XYZ format and rotates                *
!* the system by a certain angle around the cartesian X, Y or Z axis.             *
!* Then it outputs the result in a XYZ file.                                      *
!**********************************************************************************
!* (C) May 2010 - Pierre Hirel                                                    *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 27 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
!
SUBROUTINE ROTATE_XYZ(inputfile,outputfile,rot_axis,rot_angle)
!
!
IMPLICIT NONE
CHARACTER(LEN=1):: rot_axis  ! cartesian x, y or z axis
CHARACTER(LEN=128):: file, inputfile, outputfile
CHARACTER(LEN=128):: temp, msg, comment
INTEGER:: a1, a2, a3
INTEGER:: i, NP
REAL*8:: a0
REAL*8:: H2, H3
REAL*8:: rot_angle      !angle in degrees
REAL*8,DIMENSION(4):: P
REAL*8,DIMENSION(3,3):: rot_matrix
!
!Initialize variables
 comment = 'File generated by pikashu'
i = 0
a0 = 1.d0
H2 = 0.d0
H3 = 0.d0
P(:) = 0.d0
rot_matrix(:,:) = 0.d0
DO i=1,3
  rot_matrix(i,i) = 1.d0
ENDDO
!
msg = 'debug --> Entering ROTATE_XYZ'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
WRITE(msg,5) '>>> Rotating the system by ', rot_angle,'° around ', rot_axis
CALL DISPLAY_MSG(verbosity,msg,logfile)
5 FORMAT(a27,f7.2,a9,a2)
!
!If angle is more than 360°, reduce it
DO WHILE(DABS(rot_angle)>=360.d0)
  IF(rot_angle>=360.d0)  rot_angle = rot_angle-360.d0
  IF(rot_angle<=-360.d0) rot_angle = rot_angle+360.d0
ENDDO
!
IF(rot_angle==0.d0) THEN
  msg = '..> Rotation angle is modulo zero, skipping...'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  GOTO 1000
ENDIF
!
!convert the angle into radians
rot_angle = rot_angle*pi/180.d0
!
!Define the axes
IF(rot_axis=='x' .OR. rot_axis=='X') THEN
  a1 = 1
  a2 = 2
  a3 = 3
ELSEIF(rot_axis=='y' .OR. rot_axis=='Y') THEN
  a1 = 2
  a2 = 3
  a3 = 1
ELSEIF(rot_axis=='z' .OR. rot_axis=='Z') THEN
  a1 = 3
  a2 = 1
  a3 = 2
ELSE
  msg = 'X!X ERROR: could not recognize axis: '//rot_axis
  CALL DISPLAY_MSG(3,msg,logfile)
  nerr = nerr+1
  GOTO 1000
ENDIF
!
!set the rotation matrix
rot_matrix(:,:) = 0.d0
rot_matrix(a1,a1) = 1.d0
rot_matrix(a2,a2) = DCOS(rot_angle)
rot_matrix(a2,a3) = -DSIN(rot_angle)
rot_matrix(a3,a2) = DSIN(rot_angle)
rot_matrix(a3,a3) = DCOS(rot_angle)
!
msg = 'debug --> Rotation matrix:'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,60) 'debug -->            | ',                              &
             & rot_matrix(1,1), rot_matrix(1,2), rot_matrix(1,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,60) 'debug -->  R_theta = | ',                              &
             & rot_matrix(2,1), rot_matrix(2,2), rot_matrix(2,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,60) 'debug -->            | ',                              &
             & rot_matrix(3,1), rot_matrix(3,2), rot_matrix(3,3), '|'
CALL DISPLAY_MSG(verbosity,msg,logfile)
60 FORMAT(a23,3(f10.5,2X),a1)
!
!
!
100 CONTINUE
!Read input file
file = inputfile
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!Read extended info at end of file
DO
  READ(30,*,ERR=110,END=110) temp
  IF(temp(1:4)=='alat') THEN
    READ(30,*,ERR=800,END=800) a0
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(30,*,ERR=800,END=800) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=800,END=800) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=800,END=800) H(3,1), H(3,2), H(3,3)
    msg = 'debug --> Found supercell vectors'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    Hset=.TRUE.
  ENDIF
  !IF(temp(1:4)=='Redu'.OR.temp(1:4)=='redu') reduced=.TRUE.
  !IF(temp(1:4)=='Cart'.OR.temp(1:4)=='cart') reduced=.FALSE.
ENDDO
!
110 CONTINUE
H(:,:) = a0*H(:,:)
!
!
!
200 CONTINUE
REWIND(30)
OPEN(UNIT=31,FILE='pikashu_rotate.xyz',FORM='FORMATTED',STATUS='UNKNOWN')
READ(30,*) NP
READ(30,'(a128)') comment
WRITE(31,*) NP
WRITE(31,*) TRIM(comment)
!
!Rotate the atomic positions
!P(a1) is untouched
DO i=1,NP
  READ(30,*,ERR=800,END=800) P(4), P(1), P(2), P(3)
  H2 = P(a2)
  H3 = P(a3)
  P(a2) = H2*rot_matrix(a2,a2) + H3*rot_matrix(a2,a3)
  P(a3) = H2*rot_matrix(a3,a2) + H3*rot_matrix(a3,a3)
  WRITE(31,210) P(4), P(1), P(2), P(3)
ENDDO
210 FORMAT(f6.2,X,3(f10.6,2x))
!
!Rotate the base vectors of the system
!The H(:,1) are left untouched
H2 = H(a1,a2)
H3 = H(a1,a3)
H(a1,a2) = H2*rot_matrix(a2,a2) + H3*rot_matrix(a2,a3)
H(a1,a3) = H2*rot_matrix(a3,a2) + H3*rot_matrix(a3,a3)
H2 = H(a2,a2)
H3 = H(a2,a3)
H(a2,a2) = H2*rot_matrix(a2,a2) + H3*rot_matrix(a2,a3)
H(a2,a3) = H2*rot_matrix(a3,a2) + H3*rot_matrix(a3,a3)
H2 = H(a3,a2)
H3 = H(a3,a3)
H(a3,a2) = H2*rot_matrix(a2,a2) + H3*rot_matrix(a2,a3)
H(a3,a3) = H2*rot_matrix(a3,a2) + H3*rot_matrix(a3,a3)
!
!Write extended info at the end of XYZ file
WRITE(31,'(a4)') 'alat'
WRITE(31,'(a3)') '1.0'
WRITE(31,'(a9)') 'supercell'
WRITE(31,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(31,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(31,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
!
!
!
300 CONTINUE
CLOSE(30)
CLOSE(31)
!Create the outputfile
CALL SYSTEM('cp -rf pikashu_rotate.xyz '//outputfile)
msg = '..> System was rotated.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading '//TRIM(file)
CALL DISPLAY_MSG(3,msg,logfile)
WRITE(msg,*) '    This apparently occured while trying to read atom ', i
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
1000 CONTINUE
IF(verbosity<4) CALL SYSTEM('rm -f pikashu_rotate.xyz')
!
!
END SUBROUTINE ROTATE_XYZ
!
!
END MODULE rotate
