MODULE shift
!
!**********************************************************************************
!*  SHIFT                                                                         *
!**********************************************************************************
!* This module reads a file in the extended XYZ format and                        *
!* shifts atoms that are above (or below) a given plane.                          *
!* Then it outputs the result in a XYZ file.                                      *
!**********************************************************************************
!* (C) March 2010 - Pierre Hirel                                                  *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 10 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
!
SUBROUTINE SHIFT_XYZ(inputfile,outputfile,shift_dir,shift_dist,shift_axis,shift_tau1,shift_tau2)
!
!
IMPLICIT NONE
CHARACTER(LEN=1):: shift_axis     !(x, y or z)
CHARACTER(LEN=5):: shift_dir      !'above' or 'below'
CHARACTER(LEN=128):: file, inputfile, outputfile
CHARACTER(LEN=128):: temp, msg, comment
INTEGER:: a1, a2, a3
INTEGER:: shiftdir       !-1=below; +1=above
INTEGER:: i, NP, NPshifted
REAL*8:: a0
REAL*8,DIMENSION(4):: P
REAL*8:: shift_dist       !distance of the "cut plane" to the origin
REAL*8:: shift_tau1, shift_tau2  !shift vector
!
!Initialize variables
i = 0
shiftdir = 0
 comment = 'File generated by pikashu'
!
msg = 'debug --> Entering SHIFT_XYZ'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!If shift is zero then skip
IF(shift_tau1==0.d0 .AND. shift_tau2==0.d0) GOTO 1000
!
!Initialize variables
a0 = 1.d0        !If lattice parameter is not specified, default has to be 1
NPshifted = 0    !Number of shifted atoms
!
IF(shift_dir=='above') THEN
  shiftdir = 1
ELSEIF(shift_dir=='below') THEN
  shiftdir = -1
ELSE
  WRITE(msg,*) '/!\ ERROR: unknown direction '//TRIM(shift_dir)
  CALL DISPLAY_MSG(verbosity,msg,logfile)
ENDIF
!
!Define the axes
IF(shift_axis=='x' .OR. shift_axis=='X') THEN
  a1 = 1
  a2 = 2
  a3 = 3
ELSEIF(shift_axis=='y' .OR. shift_axis=='Y') THEN
  a1 = 2
  a2 = 3
  a3 = 1
ELSEIF(shift_axis=='z' .OR. shift_axis=='Z') THEN
  a1 = 3
  a2 = 1
  a3 = 2
ENDIF
!
!
WRITE(msg,50) '>>> Shifting atoms ', TRIM(shift_dir), shift_dist, 'A along ', &
           & shift_axis, ' by (', shift_tau1, shift_tau2, ')'
CALL DISPLAY_MSG(verbosity,msg,logfile)
50 FORMAT(a19,a5,f8.4,a8,a1,a5,2f8.4,a1)
!
WRITE(msg,*) 'debug --> a1, a2, a3: ', a1, a2, a3
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
!
100 CONTINUE
file = inputfile
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!Read extended info at end of file
DO
  READ(30,*,ERR=200,END=200) temp
  IF(temp(1:4)=='alat') THEN
    READ(30,*,ERR=800,END=800) a0
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(30,*,ERR=800,END=800) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=800,END=800) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=800,END=800) H(3,1), H(3,2), H(3,3)
    msg = 'debug --> Found supercell vectors'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    Hset=.TRUE.
  ENDIF
  !IF(temp(1:4)=='Redu'.OR.temp(1:4)=='redu') reduced=.TRUE.
  !IF(temp(1:4)=='Cart'.OR.temp(1:4)=='cart') reduced=.FALSE.
ENDDO
!
H(:,:) = a0*H(:,:)
!
!
!
200 CONTINUE
REWIND(30)
OPEN(UNIT=31,FILE='pikashu_shift.xyz',FORM='FORMATTED',STATUS='UNKNOWN')
!Read header
READ(30,*,ERR=800,END=800) NP
WRITE(31,*) NP
READ(30,'(a128)',ERR=800,END=800) comment
WRITE(31,*) TRIM(comment)
!
!Read positions, apply shift and write it in 'pikashu_shift.xyz'
P(:) = 0.d0
DO i=1,NP
  READ(30,*,ERR=800,END=800) P(4), P(1), P(2), P(3)
  IF( shiftdir>0 .AND. P(a1)>shift_dist  .OR.                       &
    & shiftdir<0 .AND. P(a1)<shift_dist          ) THEN
    NPshifted = NPshifted+1
    P(a2) = P(a2) + shift_tau1
    P(a3) = P(a3) + shift_tau2
  ENDIF
  WRITE(31,210) P(4), P(1), P(2), P(3)
ENDDO
210 FORMAT(f6.2,X,3(f10.6,2x))
!
!Write extended info at the end of file
WRITE(31,'(a4)') 'alat'
WRITE(31,'(a3)') '1.0'
WRITE(31,'(a9)') 'supercell'
WRITE(31,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(31,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(31,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
!
!
!
300 CONTINUE
CLOSE(30)
CLOSE(31)
!Copy it to the outputfile
CALL SYSTEM('cp -rf pikashu_shift.xyz '//outputfile)
!
WRITE(msg,310) '..> Number of atoms shifted: ', NPshifted
CALL DISPLAY_MSG(verbosity,msg,logfile)
310 FORMAT(a29,i8)
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading '//TRIM(file)
CALL DISPLAY_MSG(3,msg,logfile)
WRITE(msg,*) '    This apparently occured while trying to read atom ', i
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
1000 CONTINUE
IF(verbosity<4) CALL SYSTEM('rm -f pikashu_shift.xyz')
!
!
END SUBROUTINE SHIFT_XYZ
!
!
END MODULE shift
