MODULE sort
!
!**********************************************************************************
!*  DISLOCATION                                                                   *
!**********************************************************************************
!* This module reads a file in the extended XYZ format and                        *
!* introduces a straight edge or screw dislocation.                               *
!* Then it outputs the result in a XYZ file.                                      *
!**********************************************************************************
!* (C) May 2010 - Pierre Hirel                                                    *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 07 June 2010                                     *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
!
SUBROUTINE SORT_XYZ(inputfile,outputfile,sortcol,sortorder)
!
!
IMPLICIT NONE
CHARACTER(LEN=1):: sortcol    !x, y, z or s
CHARACTER(LEN=4):: sortorder  !up or down
CHARACTER(LEN=64):: file, inputfile, outputfile
CHARACTER(LEN=128):: temp, msg, comment
INTEGER:: i, NP
INTEGER:: sortnum
REAL*8:: a0
REAL*8,DIMENSION(:,:),ALLOCATABLE:: P
!
!Initialize variables
a0 = 1.d0
 comment = 'File generated by pikashu'
!
WRITE(msg,*) 'debug --> Entering SORT_XYZ: ', sortcol, sortorder
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
SELECT CASE (sortcol(1:1))
CASE('s','S')
  sortnum = 4
CASE('x','X')
  sortnum = 1
CASE('y','Y')
  sortnum = 2
CASE('z','Z')
  sortnum = 3
END SELECT
!
temp = sortcol
IF(sortcol(1:1)=='s') THEN
  temp = 'atomic weight'
ELSE
  temp = sortcol(1:1)//' coordinate'
ENDIF
IF(sortorder=='up') temp = 'increasing '//TRIM(temp)
IF(sortorder=='down') temp = 'decreasing '//TRIM(temp)
WRITE(msg,5) '>>> Sorting atoms by ', TRIM(ADJUSTL(temp))
CALL DISPLAY_MSG(verbosity,msg,logfile)
5 FORMAT(a21,a24)
!
!
!
100 CONTINUE
file = inputfile
OPEN(UNIT=30,FILE=inputfile,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!Read extended info at end of file
DO
  READ(30,*,ERR=120,END=120) temp
  IF(temp(1:4)=='alat') THEN
    READ(30,*,ERR=800,END=800) a0
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(30,*,ERR=800,END=800) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=800,END=800) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=800,END=800) H(3,1), H(3,2), H(3,3)
    msg = 'debug --> Found supercell vectors'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    Hset=.TRUE.
  ENDIF
ENDDO
!
120 CONTINUE
!Read atomic positions from inputfile
REWIND(30)
READ(30,*,ERR=800,END=800) NP
READ(30,'(a128)',ERR=800,END=800) comment
!
!Allocate array for atomic positions
!Note that for edge dislocations we will introduce new atoms
! so we need a larger array
ALLOCATE(P(NP,4))
P(:,:) = 0.d0
!
DO i=1,NP
  READ(30,*,ERR=800,END=800) P(i,4), P(i,1), P(i,2), P(i,3)
ENDDO
CLOSE(30)
!
!
!
200 CONTINUE
IF(NP>10000) THEN
  msg = '..> This may take some time...'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
ENDIF
!Sort atoms: here we use bubble sort algorithm (subroutines.f90)
CALL BUBBLESORT(P,sortnum,sortorder)
!
!
!
300 CONTINUE
!Write positions into the outputfile
OPEN(UNIT=31,FILE=outputfile,FORM='FORMATTED',STATUS='UNKNOWN',POSITION='REWIND')
WRITE(31,*) NP
WRITE(31,*) TRIM(comment)
DO i=1,NP
  WRITE(31,310) P(i,4), P(i,1), P(i,2), P(i,3)
ENDDO
310 FORMAT(f6.2,X,3(f10.6,2x))
!Write extended info at the end of file
WRITE(31,'(a4)') 'alat'
WRITE(31,'(a3)') '1.0'
WRITE(31,'(a9)') 'supercell'
WRITE(31,'(3(f12.6,2X))') H(1,1), H(1,2), H(1,3)
WRITE(31,'(3(f12.6,2X))') H(2,1), H(2,2), H(2,3)
WRITE(31,'(3(f12.6,2X))') H(3,1), H(3,2), H(3,3)
CLOSE(31)
!
!
!
400 CONTINUE
msg = '..> Atoms were sorted.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading '//TRIM(file)
CALL DISPLAY_MSG(3,msg,logfile)
WRITE(msg,*) '    This apparently occured while trying to read atom ', i
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
1000 CONTINUE
IF(ALLOCATED(P)) DEALLOCATE(P)
IF(verbosity<4) CALL SYSTEM('rm -f pikashu_disloc.xyz')
!
!
END SUBROUTINE SORT_XYZ
!
!
!
END MODULE sort