PROGRAM pikashu
!
!**********************************************************************************
!*                                 P I K A S H U                                  *
!*            PIerre's Swiss-army Knife for handling Atomic formats               *
!*            ==------------------=------------------=-------------               *
!*                           in a Simple and HUman way                            *
!*                           -----=----------==-------                            *
!**********************************************************************************
!* This program is meant to convert many formats of files containing              *
!* atomic positions, into other formats. This program does not deal with data     *
!* such as atom velocities, potential energies, forces or whatever,               *
!* only with atomic positions.                                                    *
!* Please read the contents of /doc for a detailed description.                   *
!**********************************************************************************
!* Version of the program: see "comv.f90"                                         *
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 16 June 2010                                     *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!* Structure of the program:                                                      *
!* 1-99   : define and initialize variables                                       *
!* 100-199: read and interpret command-line options                               *
!* 200-299: deal with the different modes                                         *
!* >500   : terminate program                                                     *
!**********************************************************************************
!* Units for files (this should be consistent in all modules):                    *
!* 20:     logfile of the program                                                 *
!* 30-34:  input files                                                            *
!* 35-39:  intermediate files (internal to PIKASHU)                               *
!* 40-45:  output files                                                           *
!**********************************************************************************
!
!
!Load modules
USE comv        !common variables
USE constants   !math and physics constants
USE functions   !functions used by the program
USE messages
USE subroutines !subroutines for this program
!
!Modules managing modes
USE modes
USE mode_create
USE mode_interactive
!
!
!
!Declare variables
IMPLICIT NONE
CHARACTER(LEN=10):: create_struc
CHARACTER(LEN=3):: infileformat, outfileformat
CHARACTER(LEN=16):: helpsection
CHARACTER(LEN=32):: clarg, temp
CHARACTER(LEN=128):: outputfile
CHARACTER(LEN=128):: file1, file2
CHARACTER(LEN=128):: filefirst, filesecond
CHARACTER(LEN=128):: listfile
CHARACTER(LEN=128):: msg
CHARACTER(LEN=2),DIMENSION(20):: create_spieces
INTEGER:: i, m, ioptions
INTEGER:: strlength
REAL*8:: create_a0
REAL*8:: smass
REAL*8:: tempreal
REAL*8:: time0, time1
CALL CPU_TIME(time0)
!
!
!Initialize variables
 create_spieces(:) = ''
 options_array(:) = ''
H(:,:) = 0.d0
Hset=.FALSE.
 clarg = ''
inputfile=''
infileformat=''
outputfile=''
outfileformat=''
listfile=''
file1 = ''
file2 = ''
filefirst = ''
filesecond = ''
strlength=0
nwarn = 0
nerr = 0
!
!Set default parameters of the program
logfile='pikashu.log'
helpsection = 'general'
mode=''
overw=.FALSE.  !By default, don't overwrite files
CALL SET_OUTPUT('all',.FALSE.)
!Define the default level of verbosity:
!    0=silent; no message is ever displayed nor written in log file
!    1=all messages are written on the screen only.
!    2=all messages are written in log file only.
!    3=messages are written in log+screen.
!    4=debug, additional messages are written in log file.
!This default value should be set to 4 only for programming/debugging purposes
verbosity = 1
!
!
CALL SYSTEM('rm -f pikashu.log pikashu.xyz')
OPEN(UNIT=20,FILE=logfile,STATUS='NEW',FORM='FORMATTED')
!
!
!
100 CONTINUE
!!Read command-line options and try to understand them
i=0
ioptions = 0
!
CALL GETARG(1,clarg)
!
DO WHILE(LEN_TRIM(clarg).NE.0)
  !If the user specified an alternate logfile, we change
  !the level of verbosity to 3
  IF( logfile.NE.'pikashu.log') THEN
    CLOSE(20)
    OPEN(UNIT=20,FILE=logfile,FORM='FORMATTED',STATUS='UNKNOWN')
    IF( verbosity==0 .OR.verbosity==1 ) verbosity = 3
  ENDIF
  !
  i=i+1
  CALL GETARG(i,clarg)
  clarg = TRIM(ADJUSTL(clarg))
  IF(LEN_TRIM(clarg)==0) EXIT
  !
  WRITE(msg,'(a31,i3,a16)') 'debug --> Command-line option #',        &
       &                    i, TRIM(ADJUSTL(clarg))
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  !
  ! 'version' and 'help' are special:
  !they print something and then exit the program
  IF(clarg=='-version') THEN
    WRITE(*,*) 'PIKASHU (C) P. Hirel 2010 - Version ', version
    GOTO 1000
  ELSEIF(clarg=='-help' .OR. clarg=='-h' .OR. clarg=='help') THEN
    i=i+1
    CALL GETARG(i,clarg)
    READ(clarg,*,END=102,ERR=102) helpsection
    IF( helpsection.NE.'general' .AND. helpsection.NE.'modes' .AND.   &
      & helpsection.NE.'options'                                    ) &
      & helpsection='general'
    102 CONTINUE
    CALL DISPLAY_HELP(helpsection)
    GOTO 1000
  !
  !
  !Deal with the behavior of the program
  ELSEIF(clarg=='-log') THEN
    i=i+1
    CALL GETARG(i,clarg)
    logfile = TRIM(ADJUSTL(clarg))
    IF(logfile=='') logfile='pikashu.log'
    CLOSE(20)
    CALL CHECKFILE(logfile,'writ')
    OPEN(UNIT=20,FILE=logfile,STATUS='UNKNOWN',FORM='FORMATTED')
  ELSEIF(clarg=='-verbosity' .OR. clarg=='-v') THEN
    i=i+1
    CALL GETARG(i,clarg)
    READ(clarg,*,END=101,ERR=101) verbosity
  ELSEIF(clarg=='-overw' .OR. clarg=='-ow') THEN
    overw=.TRUE.
  !
  !
  !Deal with formats
  ELSEIF(clarg=='xyz') THEN
    outfileformat = 'xyz'
    output_xyz = .TRUE.
  ELSEIF(clarg=='cfg') THEN
    outfileformat = 'cfg'
    output_cfg = .TRUE.
  ELSEIF(clarg=='xsf') THEN
    outfileformat = 'xsf'
    output_xsf = .TRUE.
  ELSEIF(clarg=='gin') THEN
    outfileformat = 'gin'
    output_gin = .TRUE.
  ELSEIF(clarg=='poscar' .OR. clarg=='pos') THEN
    outfileformat = 'pos'
    output_poscar = .TRUE.
  ELSEIF(clarg=='lmp') THEN
    outfileformat = 'lmp'
    output_lammps = .TRUE.
  ! -- add other formats here --
  !
  !
  !Deal with special modes
  ELSEIF(clarg=='-create') THEN
    mode='create'
    i=i+1
    CALL GETARG(i,temp)
    READ(temp,*,END=101,ERR=101) create_a0
    i=i+1
    CALL GETARG(i,temp)
    READ(temp,*,END=101,ERR=101) create_struc
    DO m=1,20
      i=i+1
      CALL GETARG(i,temp)
      IF(LEN_TRIM(temp)<=2) THEN
        READ(temp,*,END=101,ERR=101) create_spieces(m)
        CALL ATOMMASS(create_spieces(m),smass)
        IF(smass==0.d0) THEN
          i=i-1
          GOTO 101
        ENDIF
      ELSE
        i=i-1
      ENDIF
    ENDDO
  ELSEIF(clarg=='-list' .OR. clarg=='-l') THEN
    mode = 'list'
    i=i+1
    CALL GETARG(i,listfile)
  ELSEIF(clarg=='-ddplot' .OR. clarg=='-dd') THEN
    mode = 'ddplot'
    i=i+1
    CALL GETARG(i,filefirst)
    i=i+1
    CALL GETARG(i,filesecond)
  ELSEIF(clarg=='-diff') THEN
    mode = 'diff'
    i=i+1
    CALL GETARG(i,file1)
    i=i+1
    CALL GETARG(i,file2)
  ELSEIF(clarg=='-merge') THEN
    mode = 'merge'
    i=i+1
    CALL GETARG(i,filefirst)
    i=i+1
    CALL GETARG(i,filesecond)
  !
  !
  !Deal with special options
  !For each option we store the option parameters in a string array
  ELSEIF(clarg=='-cut') THEN
    i=i+1
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
    !read 'above' or 'below'
    CALL GETARG(i,temp)
    temp = TRIM(ADJUSTL(temp))
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF(temp(1:5).NE.'above' .AND. temp(1:5).NE.'below') GOTO 180
    !read cut distance
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
    !read cut direction (x, y or z)
    i=i+1
    CALL GETARG(i,temp)
    temp = TRIM(ADJUSTL(temp))
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF(temp(1:1).NE.'x' .AND. temp(1:1).NE.'y' .AND. temp(1:1).NE.'z') GOTO 180
    msg = 'debug --> option array: '//TRIM(options_array(ioptions))
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  !
  ELSEIF(clarg=='-expand' .OR. clarg=='-e') THEN
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
    DO m=1,3
      i=i+1
      CALL GETARG(i,temp)
      options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
      READ(temp,*,END=180,ERR=180) tempreal
    ENDDO
    msg = 'debug --> option array: '//TRIM(options_array(ioptions))
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  !
  ELSEIF(clarg=='-deform' .OR. clarg=='-def') THEN
    i=i+1
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
    !read deformation direction (x, y or z)
    CALL GETARG(i,temp)
    temp = TRIM(ADJUSTL(temp))
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF(temp(1:1).NE.'x' .AND. temp(1:1).NE.'y' .AND. temp(1:1).NE.'z') GOTO 180
    !read deformation %
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
    !read Poisson ratio
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
    msg = 'debug --> option array: '//TRIM(options_array(ioptions))
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  !
  ELSEIF(clarg=='-shift') THEN
    i=i+1
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
    !read part to shift ('above' or 'below')
    CALL GETARG(i,temp)
    temp = TRIM(ADJUSTL(temp))
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF(temp(1:5).NE.'above' .AND. temp(1:5).NE.'below') GOTO 180
    !read distance of plane of shift
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
    !read axis perpendicular to the plane of shift (x, y or z)
    i=i+1
    CALL GETARG(i,temp)
    temp = TRIM(ADJUSTL(temp))
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF(temp(1:1).NE.'x' .AND. temp(1:1).NE.'y' .AND. temp(1:1).NE.'z') GOTO 180
    !read tau1
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
    !read tau2
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,ENd=180,ERR=180) tempreal
    msg = 'debug --> option array: '//TRIM(options_array(ioptions))
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  !
  ELSEIF(clarg=='-rotate' .OR. clarg=='-rot') THEN
    i=i+1
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
    !read the axis of rotation (x, y or z)
    CALL GETARG(i,temp)
    temp = TRIM(ADJUSTL(temp))
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF(temp(1:1).NE.'x' .AND. temp(1:1).NE.'y' .AND. temp(1:1).NE.'z') GOTO 180
    !read the angle of rotation
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
  !
  ELSEIF(clarg=='-dislocation' .OR. clarg=='-disloc') THEN
    i=i+1
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
    !read the type of dislocation (edge or screw)
    CALL GETARG(i,temp)
    temp = TRIM(ADJUSTL(temp))
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF(temp.NE.'screw' .AND. temp.NE.'edge') GOTO 180
    !read the dislocation line direction (x, y or z)
    i=i+1
    CALL GETARG(i,temp)
    temp = TRIM(ADJUSTL(temp))
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF(temp(1:1).NE.'x' .AND. temp(1:1).NE.'y' .AND. temp(1:1).NE.'z') GOTO 180
    !read the length of Burgers vector
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
    !read Poisson's ratio
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
    !read first coordinate of disloc.
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
    !read second coordinate of disloc.
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    READ(temp,*,END=180,ERR=180) tempreal
  !
  ELSEIF(clarg=='-alignx') THEN
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
  !
  !
  ELSEIF(clarg=='-rebox') THEN
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
  !
  ELSEIF(clarg=='-sort') THEN
    ioptions = ioptions+1
    options_array(ioptions) = TRIM(clarg)
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF( temp(1:1).NE.'x' .AND. temp(1:1).NE.'y' .AND.                      &
      & temp(1:1).NE.'z' .AND. temp(1:1).NE.'s'        ) GOTO 180
    i=i+1
    CALL GETARG(i,temp)
    options_array(ioptions) = TRIM(options_array(ioptions))//' '//TRIM(temp)
    IF( temp.NE.'up' .AND. temp.NE.'down' ) GOTO 180
  !
  ! -- add other options here --
  !
  !
  !If it is none of the above, we assume it is some file name
  ELSEIF(file1=='') THEN
    IF(mode=='') mode='normal'
    WRITE(file1,*) TRIM(clarg)
  ELSEIF(file2=='') THEN
    IF(mode=='') mode='normal'
    WRITE(file2,*) TRIM(clarg)
  !
  !
  !And the rest we do not understand
  ELSE
    msg = '/!\ WARNING: Unrecognized command-line argument: '//TRIM(clarg)
    nwarn = nwarn+1
    CALL DISPLAY_MSG(verbosity,msg,logfile)
  ENDIF
  101 CONTINUE
ENDDO
!
!
!
!Print a nice message
CALL DISPLAY_HEADER()
!
msg = 'debug --> file1, file2:'//TRIM(file1)//', '//TRIM(file2)
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = 'debug --> OPTIONS ARRAY:'
CALL DISPLAY_MSG(verbosity,msg,logfile)
ioptions = 1
DO WHILE( options_array(ioptions).NE.'' )
  msg = 'debug --> '//TRIM(options_array(ioptions))
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  ioptions = ioptions+1
ENDDO
GOTO 200
!
!
180 CONTINUE
nerr=nerr+1
CALL DISPLAY_HEADER()
msg = 'X!X ERROR: non-conform statement in option: '//TRIM(options_array(ioptions))
CALL DISPLAY_MSG(3,msg,logfile)
READ(options_array(ioptions),*) temp
CALL DISPLAY_HELP(temp)
GOTO 500
!
!
!
200 CONTINUE
!If list or ddplot mode, we have to setup the files correctly
IF(mode=='list') THEN
  file1=listfile
ENDIF
!
!
! Then we deal with the different modes:
!
!If no command-line argument was specified at all, then
!we enter interactive mode
IF(mode=='') THEN
  mode = 'interactive'
  CALL INTERACT()
!
!Special case for create mode
ELSEIF(mode=='create') THEN
  CALL CREATE_CELL(create_a0,create_struc,create_spieces,file1)
!
!Otherwise we enter RUN_MODE
ELSE
  CALL RUN_MODE(file1,file2,filefirst,filesecond,outfileformat)
ENDIF
!
!
!
500 CONTINUE
!Terminating program
msg = "\o/ Program terminated successfully!"
CALL DISPLAY_MSG(verbosity,msg,logfile)
WRITE(msg,501) '    Warnings: ', nwarn, '; Errors: ', nerr
CALL DISPLAY_MSG(verbosity,msg,logfile)
501 FORMAT(a14,i3,a10,i2)
!
!
!
1000 CONTINUE
!Erase temporary file(s)
IF(verbosity<4) CALL SYSTEM('rm -f pikashu.xyz')
IF(verbosity<2 .OR. logfile.NE.'pikashu.log') CALL SYSTEM('rm -f pikashu.log')
!
CALL CPU_TIME(time1)
time1 = time1-time0
WRITE(msg,*) 'debug --> Total time (s):', time1
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
!
END PROGRAM pikashu
