MODULE readin
!
!**********************************************************************************
!*  READIN                                                                        *
!**********************************************************************************
!* This module reads a single file and converts it to an XYZ file.                *
!**********************************************************************************
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 25 May 2010                                      *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
!Load modules
USE comv
USE constants
USE functions
USE messages
USE files
USE subroutines
USE guess_form
!
!Modules managing input files
USE in_gulp_gin
USE in_cfg
USE in_xyz
USE in_xsf
USE in_vasp_poscar
! -- add other formats here --
!
!
!
CONTAINS
!
SUBROUTINE READ_AFF(inputfile,outputfile)
!
!
!
!Declare variables
IMPLICIT NONE
CHARACTER(LEN=3):: infileformat, outfileformat
CHARACTER(LEN=128):: inputfile, outputfile
CHARACTER(LEN=128):: msg
INTEGER:: NP
INTEGER:: strlength
!
!
!Initialize variables
NP = 0
infileformat=''
strlength=0
!
!If the interactive mode runs, then we must use the
!'outfileformat' defined previously
!Else, the output file formats are defined by the
!global logical variables (output_xyz, output_cfg, etc.)
!so we have to reset the 'outfileformat' variable
IF(mode.NE.'interactive') outfileformat = ''
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  CHECK THE FILES, IDENTIFY FORMATS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
100 CONTINUE
msg = 'debug --> CONVERT_AFF'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = 'debug --> Inputfile, outputfile: '//TRIM(inputfile)//', '//TRIM(outputfile)
CALL DISPLAY_MSG(verbosity,msg,logfile)
!If no input file was specified, or if it does not exist, then prompt the user
msg = 'debug --> Check if file exists: '//TRIM(inputfile)
CALL DISPLAY_MSG(verbosity,msg,logfile)
CALL CHECKFILE(inputfile,'read')
!
120 CONTINUE
!If no output file was specified, prompt the user
IF(LEN_TRIM(outputfile)==0) THEN
  nwarn = nwarn+1
  msg = '/!\ WARNING: No output file was specified.'
  CALL DISPLAY_MSG(3,msg,logfile)
  msg = '(°< Please provide a name for the output file:'
  CALL DISPLAY_MSG(3,msg,logfile)
  READ(*,*) outputfile
  CALL CHECKFILE(outputfile,'writ')
  CALL GUESS_FORMAT(outputfile,outfileformat)
ENDIF
!
!
130 CONTINUE
msg = '>>> Opening the input file: '//TRIM(ADJUSTL(inputfile))
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!!Open input file
!!and find out what is its format
!Note: the format of the input file is not guessed only
!     from its extension, but also from its contents; see
!     the routine GUESS_FORMAT for more details.
CALL GUESS_FORMAT(inputfile,infileformat)
msg = '..> Input file format: '//infileformat
CALL DISPLAY_MSG(verbosity,msg,logfile)
IF(infileformat=='xxx') THEN
  nwarn=nwarn+1
  msg = '/!\ WARNING: I could not guess the format of the input file!'
  CALL DISPLAY_MSG(3,msg,logfile)
  msg = '    Maybe it is a format unsupported by pikashu yet?'
  CALL DISPLAY_MSG(3,msg,logfile)
  msg = '    Anyway it appears I cannot help, I will skip this file.'
  CALL DISPLAY_MSG(3,msg,logfile)
  GOTO 1000
ENDIF
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  READ INPUT FILE
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
200 CONTINUE
msg = '>>> Reading input file: '//TRIM(ADJUSTL(inputfile))
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!!Then, read input file
! by calling the module corresponding to the input format
SELECT CASE(infileformat)
CASE('xyz')
  CALL READ_XYZ(inputfile)
CASE('cfg')
  CALL READ_CFG(inputfile)
CASE('gin')
  CALL READ_GIN(inputfile)
CASE('xsf')
  CALL READ_XSF(inputfile)
CASE('pos')
  CALL READ_POSCAR(inputfile)
!
! -- add other formats here --
!
CASE DEFAULT
  GOTO 800
END SELECT
!
IF(nerr>=1) GOTO 800
msg = '..> Input file was read successfully.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
GOTO 1000
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  ERROR MESSAGES
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
800 CONTINUE
msg = 'X!X ERROR: there were errors while reading the file:'//TRIM(inputfile)
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
!
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  TERMINATE MODULE
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
1000 CONTINUE
IF(outputfile.NE.'pikashu.xyz') CALL SYSTEM('mv pikashu.xyz '//outputfile)
!
!
END SUBROUTINE READ_AFF
!
!
END MODULE readin
