MODULE subroutines
!
!**********************************************************************************
!*  SUBROUTINES                                                                   *
!**********************************************************************************
!* This module contains general subroutines dealing                               *
!* with files, manipulation of strings, arrays and so on.                         *
!**********************************************************************************
!* (C) Feb. 2010 - Pierre Hirel                                                   *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 07 June 2010                                     *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!* List of subroutines in this module:                                            *
!* WRITE_XYZFOOT       writes the footer of extended XYZ files                    *
!* NAME_OUTFILE        names an output file based on the name of an input file    *
!* CHECKFILE           checks if a file exists and if so, prompt about it         *
!* BUBBLESORT          sorts an array                                             *
!* DO_STATS            do some statistics on a 1-dimensional array                *
!* INVMAT              inverts a matrix                                           *
!* CONVMAT             converts conventional vectors into a matrix                *
!* INDEX_PLANE         find the indices of a plane from a string                  *
!* ATOMMASS            gives the mass of an atom providing its symbol             *
!* ATOMSPIECES         gives an atom symbol providing its mass                    *
!**********************************************************************************
!
!
USE comv
USE constants
USE functions
USE messages
!
IMPLICIT NONE
!
!
CONTAINS
!
!
!
!********************************************************
! WRITE_XYZFOOT
! This subroutine writes the footer of extended
! XYZ files
!********************************************************
SUBROUTINE WRITE_XYZFOOT(xyzfile,a0,xyzform)
!
IMPLICIT NONE
CHARACTER(LEN=10):: xyzform  !cartesian or fractional
CHARACTER(LEN=64):: xyzfile
REAL*8:: a0
!
OPEN(UNIT=50,FILE=xyzfile,STATUS='OLD',FORM='FORMATTED',POSITION='APPEND')
!
WRITE(50,'(a4)') 'alat'
WRITE(50,'(f12.6)') a0
WRITE(50,'(a9)') 'supercell'
WRITE(35,'(3(f12.6,X))') H(1,1), H(1,2), H(1,3)
WRITE(35,'(3(f12.6,X))') H(2,1), H(2,2), H(2,3)
WRITE(35,'(3(f12.6,X))') H(3,1), H(3,2), H(3,3)
WRITE(35,*) TRIM(xyzform)//' coordinates'
!
CLOSE(50)
!
END SUBROUTINE WRITE_XYZFOOT
!
!
!********************************************************
!  CHECKFILE
!  This subroutine checks if a file exists. Next
!  action depends on the file status:
!  - if the file has to be read but doesn't exist, then
!    prompt the user for another file to read;
!  - if the file has to be written but already exists,
!    then prompt the user whether to overwrite the file
!    or chose another name.
!********************************************************
!
SUBROUTINE CHECKFILE(filename,fstatus)
!
IMPLICIT NONE
CHARACTER(LEN=1):: test
CHARACTER(LEN=4):: fstatus   !file status; 'read' or 'writ'
CHARACTER(LEN=128):: filename, temp
LOGICAL:: fileexists
!
INQUIRE(FILE=filename,EXIST=fileexists)
IF(fstatus=='writ') THEN   !if we want to write the file
  DO WHILE(fileexists)
    WRITE(*,*) '(°< This file already exists: ', TRIM(filename)
    WRITE(*,*) '    Do you want to overwrite it (y/n)?'
    READ(*,*) test
    IF(test.NE.'y') THEN
      WRITE(*,*) '(°< Enter a name for the file to write:'
      READ(*,*) filename
      INQUIRE(FILE=filename,EXIST=fileexists)
    ELSE
      WRITE(*,*) '..> OK, I will overwrite ', filename
      CALL SYSTEM('rm -rf '//filename)
      fileexists = .FALSE.
    ENDIF
  ENDDO
!
ELSE  !if we want to read the file
  DO WHILE(.NOT.fileexists)
    WRITE(*,*) '(°< This file does not exist: ', TRIM(filename)
    WRITE(*,*) '    Please provide the name of an existing file:'
    WRITE(*,*) "    (type 'ls' for a list of files of current directory)"
    READ(*,*) temp
    IF(TRIM(ADJUSTL(temp))=='ls') THEN
      CALL SYSTEM('ls')
      WRITE(*,*) '(°< Enter the name of an existing file:'
      READ(*,*) filename
    ELSE
      filename = TRIM(ADJUSTL(temp))
    ENDIF
    INQUIRE(FILE=filename,EXIST=fileexists)
  ENDDO
!
ENDIF
!
END SUBROUTINE CHECKFILE
!
!
!********************************************************
! NAME_OUTFILE
! This subroutine finds a name for an output file
! based on the name of an input file and the extension
! for the output file.
! For instance if an input file 'test' is to be converted
! to .def, the output file will be 'test.def'.
! But if the name of the input file is 'test.abc',
! the name of the output file must be 'test.def',
! and not 'test.abc.def'.
!********************************************************
SUBROUTINE NAME_OUTFILE(inputfile,outputfile,outfileformat)
!
IMPLICIT NONE
CHARACTER(LEN=3):: outfileformat
CHARACTER(LEN=64):: test, inputfile,outputfile
INTEGER:: strlength
!
test = TRIM(ADJUSTL(inputfile))
strlength = LEN_TRIM(test)
!Test if the input file has an extension
IF(test(strlength:strlength)=='.') test=test(1:strlength-1)
IF(test(strlength-3:strlength-3)=='.') test=test(1:strlength-4)
outputfile = TRIM(ADJUSTL(test))//'.'//TRIM(ADJUSTL(outfileformat))
!
END SUBROUTINE NAME_OUTFILE
!
!
!********************************************************
! BUBBLESORT
! This subroutine sorts a MxN array by increasing or
! decreasing values of its column 'col'.
! It uses the so-called "bubble-sort" algorithm.
! IMPORTANT NOTE: this algorithm swaps entire columns
! of an array, up to N*(N-1) times (N=number of columns),
! so you may find it highly unefficient for very
! large arrays.
!********************************************************
SUBROUTINE BUBBLESORT(A,col,order)
!
IMPLICIT NONE
CHARACTER(LEN=4):: order  !up or down
INTEGER:: i, j
INTEGER:: col, asize
REAL*8,DIMENSION(:,:),INTENT(INOUT):: A
REAL*8,DIMENSION(SIZE(A,2)) :: col_value
!
asize = SIZE(A,1)
 col_value(:) = 0.d0
!
IF(order=='up') THEN
  DO j=1,asize
    DO i=j+1,asize
      !If element i is smaller than element j, we swap them
      IF( A(i,col) < A(j,col) ) THEN
        col_value(:) = A(i,:)
        A(i,:) = A(j,:)
        A(j,:) = col_value(:)
      ENDIF
    ENDDO
  ENDDO
!
ELSE
  DO j=1,asize
    DO i=j+1,asize
      !If element i is greater than element j, we swap them
      IF( A(i,col) > A(j,col) ) THEN
        col_value(:) = A(i,:)
        A(i,:) = A(j,:)
        A(j,:) = col_value(:)
      ENDIF
    ENDDO
  ENDDO
ENDIF
!
END SUBROUTINE BUBBLESORT
!
!
!********************************************************
! DO_STATS
! This subroutine performs some simple statistics
! on the numbers contained in a 1-dimensional array.
!********************************************************
SUBROUTINE DO_STATS(array,mi,M,A,D,S)
!
IMPLICIT NONE
INTEGER:: a_size, i
REAL*8,INTENT(OUT):: mi, M, A, D, S
REAL*8,DIMENSION(:),INTENT(IN):: array
!
a_size = SIZE(array)
A = 0.d0
D = 0.d0
S = 0.d0
!
!Find the minimum (mi) and maximum (M)
!values in the array
mi = MINVAL(array)
M  = MAXVAL(array)
!
!Calculate the average (A) of all values in the array
DO i=1,a_size
  A = A+array(i)
ENDDO
A = A/REAL(a_size)
!
!Calculate average absolute deviation (D)
!and standard deviation (S)
DO i=1,a_size
  D = D + DABS(array(i)-A)
  S = S + (array(i)-A)**2.d0
ENDDO
D = D/REAL(a_size)
S = DSQRT( S/REAL(a_size) )
!
END SUBROUTINE DO_STATS
!
!
!********************************************************
! INVMAT
! This subroutine inverts a 3x3 matrix H
! into the matrix G.
!********************************************************
SUBROUTINE INVMAT(H,G)
!
IMPLICIT NONE
REAL*8, DIMENSION(3,3) :: H, G
REAL*8 :: det
!
det =   H(1,1)*H(2,2)*H(3,3) - H(1,1)*H(3,2)*H(2,3) &
    & - H(2,1)*H(1,2)*H(3,3) + H(2,1)*H(3,2)*H(1,3) &
    & + H(3,1)*H(1,2)*H(2,3) - H(3,1)*H(2,2)*H(1,3)
!
G(1,1) = (H(2,2)*H(3,3) - H(2,3)*H(3,2))/det
G(2,1) = (H(2,3)*H(3,1) - H(2,1)*H(3,3))/det
G(3,1) = (H(2,1)*H(3,2) - H(2,2)*H(3,1))/det
!
G(1,2) = (H(3,2)*H(1,3) - H(3,3)*H(1,2))/det
G(2,2) = (H(3,3)*H(1,1) - H(3,1)*H(1,3))/det
G(3,2) = (H(3,1)*H(1,2) - H(3,2)*H(1,1))/det
!
G(1,3) = (H(1,2)*H(2,3) - H(1,3)*H(2,2))/det
G(2,3) = (H(1,3)*H(2,1) - H(1,1)*H(2,3))/det
G(3,3) = (H(1,1)*H(2,2) - H(1,2)*H(2,1))/det
!
END SUBROUTINE INVMAT
!
!
!********************************************************
!  CONVMAT
!  This subroutine converts conventional vectors
!  defined by a b c alpha beta gamma,
!  into a lower triangular matrix:
!        |  H(1,1)    0       0     |
!   H =  |  H(2,1)  H(2,2)    0     |
!        |  H(3,1)  H(3,2)  H(3,3)  |
!********************************************************
!
SUBROUTINE CONVMAT(a,b,c,alpha,beta,gamma,H)
!
IMPLICIT NONE
REAL*8:: a, b, c, alpha, beta, gamma
REAL*8, DIMENSION(3,3):: H
!
H(:,:) = 0.d0
H(1,1) = a
H(2,1) = b*DCOS(gamma)
H(2,2) = b*DSIN(gamma)
H(3,1) = c*DCOS(beta)
H(3,2) = c*( DSIN(beta)*( DCOS(alpha)-DCOS(beta)*DCOS(gamma) )/ &
       &      (DSIN(beta)*DSIN(gamma))             )
H(3,3) = c*(DSIN(beta)*                                         &
       &     DSQRT(                                             &
       &           ( DSIN(gamma)**2.d0                          &
       &             -DCOS(beta)**2.d0 - DCOS(alpha)**2.d0      &
       &             +2.d0*DCOS(alpha)*DCOS(beta)*DCOS(gamma)   &
       &           )                                            &
       &          )/(DSIN(beta)*DSIN(gamma))                    &
       &   )
!
!
END SUBROUTINE CONVMAT
!
!
!********************************************************
!  INDEX_PLANE
!  This subroutine finds the indices of a plane
!  from a string.
!  For instance if the string reads '1-10' then it
!  has to be converted to the array (1 -1 0).
!********************************************************
!
SUBROUTINE INDEX_PLANE(planestring,planeindices)
!
IMPLICIT NONE
CHARACTER(LEN=1):: temp
CHARACTER(LEN=3):: mtemp
CHARACTER(LEN=16):: planestring
INTEGER:: i, m, msign
INTEGER,DIMENSION(3):: planeindices
!
planeindices(:) = 0
!
m = 0
mtemp = ''
msign = 1
DO i=1,16
  READ(planestring(i:i),*) temp
  IF(temp=='-') THEN
    msign = -1
  ELSEIF(temp=='+') THEN
    msign = 1
  ELSEIF(TRIM(temp).NE.'') THEN
    READ(temp,*) planeindices(m)
    planeindices(m) = REAL(msign)*planeindices(m)
    m = m+1
  ENDIF
ENDDO
!
END SUBROUTINE INDEX_PLANE
!
!
!********************************************************
!  ATOMMASS
!  This subroutine sets the mass of an atom
!  depending on its spieces.
!********************************************************
!
SUBROUTINE ATOMMASS(spieces,smass)
!
IMPLICIT NONE
CHARACTER(LEN=2):: spieces
REAL*8:: smass
!
!
SELECT CASE(spieces)
! n=1
CASE('H')
      smass=1.d0
CASE('He')
      smass=2.d0
!
! n=2
CASE('Li')
      smass=3.d0
CASE('Be')
      smass=4.d0
CASE('B')
      smass=5.d0
CASE('C')
      smass=6.d0
CASE('N')
      smass=7.d0
CASE('O')
      smass=8.d0
CASE('F')
      smass=9.d0
CASE('Ne')
      smass=10.d0
!
! n=3
CASE('Na') 
      smass=11.d0
CASE('Mg') 
      smass=12.d0
CASE('Al') 
      smass=13.d0
CASE('Si') 
      smass=14.d0
CASE('P') 
      smass=15.d0
CASE('S') 
      smass=16.d0
CASE('Cl') 
      smass=17.d0
CASE('Ar') 
      smass=18.d0
!
! n=4
CASE('K') 
      smass=19.d0
CASE('Ca') 
      smass=20.d0
CASE('Sc') 
      smass=21.d0
CASE('Ti') 
      smass=22.d0
CASE('V') 
      smass=23.d0
CASE('Cr') 
      smass=24.d0
CASE('Mn') 
      smass=25.d0
CASE('Fe') 
      smass=26.d0
CASE('Co') 
      smass=27.d0
CASE('Ni') 
      smass=28.d0
CASE('Cu') 
      smass=29.d0
CASE('Zn') 
      smass=30.d0
CASE('Ga') 
      smass=31.d0
CASE('Ge') 
      smass=32.d0
CASE('As') 
      smass=33.d0
CASE('Se') 
      smass=34.d0
CASE('Br') 
      smass=35.d0
CASE('Kr') 
      smass=36.d0
!
! n=5
CASE('Rb') 
      smass=37.d0
CASE('Sr') 
      smass=38.d0
CASE('Y') 
      smass=39.d0
CASE('Zr') 
      smass=40.d0
CASE('Nb') 
      smass=41.d0
CASE('Mo') 
      smass=42.d0
CASE('Tc') 
      smass=43.d0
CASE('Ru') 
      smass=44.d0
CASE('Rh') 
      smass=45.d0
CASE('Pd') 
      smass=46.d0
CASE('Ag') 
      smass=47.d0
CASE('Cd') 
      smass=48.d0
CASE('In') 
      smass=49.d0
CASE('Sn') 
      smass=50.d0
CASE('Sb') 
      smass=51.d0
CASE('Te') 
      smass=52.d0
CASE('I') 
      smass=53.d0
CASE('Xe') 
      smass=54.d0
!
! n=6
CASE('Cs') 
      smass=55.d0
CASE('Ba') 
      smass=56.d0
! Lanthanides
CASE('La') 
      smass=57.d0
CASE('Ce') 
      smass=58.d0
CASE('Pr') 
      smass=59.d0
CASE('Nd') 
      smass=60.d0
CASE('Pm') 
      smass=61.d0
CASE('Sm') 
      smass=62.d0
CASE('Eu') 
      smass=63.d0
CASE('Gd') 
      smass=64.d0
CASE('Tb') 
      smass=65.d0
CASE('Dy') 
      smass=66.d0
CASE('Ho') 
      smass=67.d0
CASE('Er') 
      smass=68.d0
CASE('Tm') 
      smass=69.d0
CASE('Yb') 
      smass=70.d0
CASE('Lu') 
      smass=71.d0
! End of Lanthanides
CASE('Hf') 
      smass=72.d0
CASE('Ta') 
      smass=73.d0
CASE('W') 
      smass=74.d0
CASE('Re') 
      smass=75.d0
CASE('Os') 
      smass=76.d0
CASE('Ir') 
      smass=77.d0
CASE('Pt') 
      smass=78.d0
CASE('Au') 
      smass=79.d0
CASE('Hg') 
      smass=80.d0
CASE('Tl') 
      smass=81.d0
CASE('Pb') 
      smass=82.d0
CASE('Bi') 
      smass=83.d0
CASE('Po') 
      smass=84.d0
CASE('At') 
      smass=85.d0
CASE('Rn') 
      smass=86.d0
!
! n=7
CASE('Fr') 
      smass=87.d0
CASE('Ra') 
      smass=88.d0
! Actinides
CASE('Ac') 
      smass=89.d0
CASE('Th') 
      smass=90.d0
CASE('Pa') 
      smass=91.d0
CASE('U') 
      smass=92.d0
CASE('Np') 
      smass=93.d0
CASE('Pu') 
      smass=94.d0
CASE('Am') 
      smass=95.d0
CASE('Cm') 
      smass=96.d0
CASE('Bk') 
      smass=97.d0
CASE('Cf') 
      smass=98.d0
CASE('Es') 
      smass=99.d0
CASE('Fm') 
      smass=100.d0
CASE('Md') 
      smass=101.d0
CASE('No') 
      smass=102.d0
CASE('Lr') 
      smass=103.d0
! End of actinides
CASE('Rf') 
      smass=104.d0
CASE('Db') 
      smass=105.d0
CASE('Sg') 
      smass=106.d0
CASE('Bh') 
      smass=107.d0
CASE('Hs') 
      smass=108.d0
CASE('Mt') 
      smass=109.d0
CASE('Ds') 
      smass=110.d0
CASE('Rg') 
      smass=111.d0
CASE('Cn') 
      smass=112.d0
!
CASE DEFAULT
  !If the spieces is not recognized
  smass=0.d0
END SELECT
!
END SUBROUTINE ATOMMASS
!
!
!
!********************************************************
!  ATOMSPIECES
!  This subroutine sets the spieces of an atom
!  depending on its mass.
!********************************************************
!
SUBROUTINE ATOMSPIECES(smass,spieces)
!
IMPLICIT NONE
CHARACTER(LEN=2):: spieces
INTEGER:: smint
REAL*8:: smass
!
smint = INT(smass)
!
SELECT CASE(smint)
! n=1
CASE(1)
      spieces='H'
CASE(2)
      spieces='He'
!
! n=2
CASE(3)
      spieces='Li'
CASE(4)
      spieces='Be'
CASE(5)
      spieces='B'
CASE(6)
      spieces='C'
CASE(7)
      spieces='N'
CASE(8)
      spieces='O'
CASE(9)
      spieces='F'
CASE(10)
      spieces='Ne'
!
! n=3
CASE(11)
      spieces='Na'
CASE(12)
      spieces='Mg'
CASE(13)
      spieces='Al'
CASE(14)
      spieces='Si'
CASE(15)
      spieces='P'
CASE(16)
      spieces='S'
CASE(17)
      spieces='Cl'
CASE(18)
      spieces='Ar'
!
! n=4
CASE(19)
      spieces='K'
CASE(20)
      spieces='Ca'
CASE(21)
      spieces='Sc'
CASE(22)
      spieces='Ti'
CASE(23)
      spieces='V'
CASE(24)
      spieces='Cr'
CASE(25)
      spieces='Mn'
CASE(26)
      spieces='Fe'
CASE(27)
      spieces='Co'
CASE(28)
      spieces='Ni'
CASE(29)
      spieces='Cu'
CASE(30)
      spieces='Zn'
CASE(31)
      spieces='Ga'
CASE(32)
      spieces='Ge'
CASE(33)
      spieces='As'
CASE(34)
      spieces='Se'
CASE(35)
      spieces='Br'
CASE(36)
      spieces='Kr'
!
! n=5
CASE(37)
      spieces='Rb'
CASE(38)
      spieces='Sr'
CASE(39)
      spieces='Y'
CASE(40)
      spieces='Zr'
CASE(41)
      spieces='Nb'
CASE(42)
      spieces='Mo'
CASE(43)
      spieces='Tc'
CASE(44)
      spieces='Ru'
CASE(45)
      spieces='Rh'
CASE(46)
      spieces='Pd'
CASE(47)
      spieces='Ag'
CASE(48)
      spieces='Cd'
CASE(49)
      spieces='In'
CASE(50)
      spieces='Sn'
CASE(51)
      spieces='Sb'
CASE(52)
      spieces='Te'
CASE(53)
      spieces='I'
CASE(54)
      spieces='Xe'
!
! n=6
CASE(55)
      spieces='Cs'
CASE(56)
      spieces='Ba'
! Lanthanides
CASE(57)
      spieces='La'
CASE(58)
      spieces='Ce'
CASE(59)
      spieces='Pr'
CASE(60)
      spieces='Nd'
CASE(61)
      spieces='Pm'
CASE(62)
      spieces='Sm'
CASE(63)
      spieces='Eu'
CASE(64)
      spieces='Gd'
CASE(65)
      spieces='Tb'
CASE(66)
      spieces='Dy'
CASE(67)
      spieces='Ho'
CASE(68)
      spieces='Er'
CASE(69)
      spieces='Tm'
CASE(70)
      spieces='Yb'
CASE(71)
      spieces='Lu'
! End of lanthanides
CASE(72)
      spieces='Hf'
CASE(73)
      spieces='Ta'
CASE(74)
      spieces='W'
CASE(75)
      spieces='Re'
CASE(76)
      spieces='Os'
CASE(77)
      spieces='Ir'
CASE(78)
      spieces='Pt'
CASE(79)
      spieces='Au'
CASE(80)
      spieces='Hg'
CASE(81)
      spieces='Tl'
CASE(82)
      spieces='Pb'
CASE(83)
      spieces='Bi'
CASE(84)
      spieces='Po'
CASE(85)
      spieces='At'
CASE(86)
      spieces='Rn'
!
! n=7
CASE(87)
      spieces='Fr'
CASE(88)
      spieces='Ra'
! Actinides
CASE(89)
      spieces='Ac'
CASE(90)
      spieces='Th'
CASE(91)
      spieces='Pa'
CASE(92)
      spieces='U'
CASE(93)
      spieces='Np'
CASE(94)
      spieces='Pu'
CASE(95)
      spieces='Am'
CASE(96)
      spieces='Cm'
CASE(97)
      spieces='Bk'
CASE(98)
      spieces='Cf'
CASE(99)
      spieces='Es'
CASE(100)
      spieces='Fm'
CASE(101)
      spieces='Md'
CASE(102)
      spieces='No'
CASE(103)
      spieces='Lr'
! End of actinides
CASE(104)
      spieces='Rf'
CASE(105)
      spieces='Db'
CASE(106)
      spieces='Sg'
CASE(107)
      spieces='Bh'
CASE(108)
      spieces='Hs'
CASE(109)
      spieces='Mt'
CASE(110)
      spieces='Ds'
CASE(111)
      spieces='Rg'
CASE(112)
      spieces='Cn'
!
CASE DEFAULT
  spieces='XX'
END SELECT
!
END SUBROUTINE ATOMSPIECES
!
!
END MODULE subroutines
