MODULE xyz_difference
!
!**********************************************************************************
!*  XYZ_DIFFERENCE                                                                *
!**********************************************************************************
!* This module reads two files in the extended XYZ format                         *
!* and computes the difference in atomic positions.                               *
!* The two XYZ files have to refer to the same system                             *
!* (i.e. same number of atoms, written in the same order)                         *
!**********************************************************************************
!* (C) March 2010 - Pierre Hirel                                                  *
!*     IZBS, Karlsruher Institut fuer Technologie                                 *
!*     Kaiserstr. 12, 76131 Karlsruhe, Germany                                    *
!*     pierre.hirel@izbs.uni-karlsruhe.de                                         *
!* Last modification: P. Hirel - 16 June 2010                                     *
!**********************************************************************************
!* This is free software, distributed under GNU/GPL version3 or greater, see:     *
!*   http://www.gnu.org/licenses/gpl.html                                         *
!* or look at the LICENSE file provided with this program.                        *
!**********************************************************************************
!
USE comv
USE constants
USE messages
USE files
USE subroutines
!
!
CONTAINS
!
!
SUBROUTINE DIFF_XYZ(firstxyz,secondxyz)
!
!
IMPLICIT NONE
CHARACTER(LEN=2):: spieces
CHARACTER(LEN=128):: firstxyz, secondxyz, file, temp, msg
LOGICAL:: reduced
INTEGER:: i, j, k, l, NP1, NP2
INTEGER:: m,n
INTEGER,DIMENSION(20):: spi_NP
REAL*8:: a0, a2
REAL*8:: H1, H2, H3
REAL*8:: norm
REAL*8:: smass
REAL*8:: x1, x2, y1, y2, z1, z2
REAL*8:: x21, y21, z21
REAL*8:: stat_min, stat_M, stat_A, stat_D, stat_S
REAL*8,DIMENSION(3,3):: H_2
REAL*8,DIMENSION(20):: spi_sp
REAL*8,DIMENSION(:),ALLOCATABLE:: spi_table
REAL*8,DIMENSION(:,:),ALLOCATABLE:: stattable
!
!
!Initialize variables
a0 = 1.d0        !If lattice parameter is not specified, default has to be 1
H(:,:) = 0.d0
H_2(:,:) = 0.d0
Hset=.FALSE.
reduced=.FALSE.  !By default, XYZ format uses cartesian coordinates
file = firstxyz
!
!
msg = '>>> Computing the difference between '//TRIM(firstxyz)//' and '//TRIM(secondxyz)
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
100 CONTINUE
!Open first XYZ file
OPEN(UNIT=30,FILE=firstxyz,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!Open second XYZ file
OPEN(UNIT=31,FILE=secondxyz,FORM='FORMATTED',STATUS='OLD',POSITION='REWIND')
!
!Check that the number of particles is the same
READ(30,*,ERR=800,END=800) NP1
READ(30,*,ERR=800,END=800) temp
READ(31,*,ERR=800,END=800) NP2
READ(31,*,ERR=800,END=800) temp
IF(NP1.NE.NP2) THEN
  msg = 'X!X ERROR: number of particles is not the same for the two systems'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  msg = '           Program will EXIT.'
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  nerr = nerr+1
  GOTO 500
ENDIF
!
!Read extended info at end of 1st file
DO
  READ(30,*,ERR=110,END=110) temp
  IF(temp(1:4)=='alat') THEN
    READ(30,*,ERR=800,END=800) a0
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(30,*,ERR=800,END=800) H(1,1), H(1,2), H(1,3)
    READ(30,*,ERR=800,END=800) H(2,1), H(2,2), H(2,3)
    READ(30,*,ERR=800,END=800) H(3,1), H(3,2), H(3,3)
    msg = 'debug --> Found supercell vectors'
    CALL DISPLAY_MSG(verbosity,msg,logfile)
    Hset=.TRUE.
  ENDIF
  IF(temp(1:4)=='Redu'.OR.temp(1:4)=='redu') reduced=.TRUE.
  IF(temp(1:4)=='Cart'.OR.temp(1:4)=='cart') reduced=.FALSE.
ENDDO
!
110 CONTINUE
H(:,:) = a0*H(:,:)
!
!Read extended info at end of 2nd file
file = secondxyz
DO
  READ(31,*,ERR=120,END=120) temp
  IF(temp(1:4)=='alat') THEN
    READ(31,*) a2
  ENDIF
  IF(temp(1:5)=='super') THEN
    READ(31,*,ERR=800,END=800) H_2(1,1), H_2(1,2), H_2(1,3)
    READ(31,*,ERR=800,END=800) H_2(2,1), H_2(2,2), H_2(2,3)
    READ(31,*,ERR=800,END=800) H_2(3,1), H_2(3,2), H_2(3,3)
  ENDIF
ENDDO
!
120 CONTINUE
H_2(:,:) = a2*H_2(:,:)
!
!If the two bases are not the same then we cannot compare atomic positions
DO m=1,3
  DO n=1,3
    IF( DABS( H_2(m,n)-H(m,n) )>=0.01d0 ) THEN
      GOTO 850
    ENDIF
  ENDDO
ENDDO
!
!
130 CONTINUE
H1=DSQRT( H(1,1)**2.d0+H(1,2)**2.d0+H(1,3)**2.d0 )
H2=DSQRT( H(2,1)**2.d0+H(2,2)**2.d0+H(2,3)**2.d0 )
H3=DSQRT( H(3,1)**2.d0+H(3,2)**2.d0+H(3,3)**2.d0 )
!
!
!
200 CONTINUE
!Check if files already exist
file = 'xyz_diff.xyz'
IF(.NOT.overw) CALL CHECKFILE(file,'writ')
file = 'xyz_diff.norm'
IF(.NOT.overw) CALL CHECKFILE(file,'writ')
file = 'xyz_diff.xsf'
IF(.NOT.overw) CALL CHECKFILE(file,'writ')
file = 'xyz_both.xsf'
IF(.NOT.overw) CALL CHECKFILE(file,'writ')
file = 'xyz_stat.txt'
IF(.NOT.overw) CALL CHECKFILE(file,'writ')
!
OPEN(UNIT=35,FILE='xyz_diff.xyz',FORM='FORMATTED',STATUS='UNKNOWN')
OPEN(UNIT=36,FILE='xyz_diff.norm',FORM='FORMATTED',STATUS='UNKNOWN')
OPEN(UNIT=37,FILE='xyz_diff.xsf',FORM='FORMATTED',STATUS='UNKNOWN')
OPEN(UNIT=38,FILE='xyz_both.xsf',FORM='FORMATTED',STATUS='UNKNOWN')
REWIND(30)
REWIND(31)
READ(30,*,ERR=800,END=800) NP1
READ(30,*,ERR=800,END=800) temp
READ(31,*,ERR=800,END=800) NP2
READ(31,*,ERR=800,END=800) temp
!
!
!
300 CONTINUE
ALLOCATE(stattable(NP1,2))
!Write header for xyz file
WRITE(35,*) NP1
WRITE(35,*) '#Atom   Cartesian coordinates of 1st file',        &
     &      '         Displacement vectors'
!
!Write header for norm file
WRITE(36,*) '#Norm of displacement vectors (A)'
!
!Write header for xsf files
WRITE(37,*) "CRYSTAL"
WRITE(37,*) "PRIMVEC"
WRITE(37,*) H(1,1), H(1,2), H(1,3)
WRITE(37,*) H(2,1), H(2,2), H(2,3)
WRITE(37,*) H(3,1), H(3,2), H(3,3)
WRITE(37,*) "CONVVEC"
WRITE(37,*) H(1,1), H(1,2), H(1,3)
WRITE(37,*) H(2,1), H(2,2), H(2,3)
WRITE(37,*) H(3,1), H(3,2), H(3,3)
WRITE(37,*) "PRIMCOORD"
WRITE(37,*) NP1, ' 1'
!
WRITE(38,*) "CRYSTAL"
WRITE(38,*) "PRIMVEC"
WRITE(38,*) H(1,1), H(1,2), H(1,3)
WRITE(38,*) H(2,1), H(2,2), H(2,3)
WRITE(38,*) H(3,1), H(3,2), H(3,3)
WRITE(38,*) "CONVVEC"
WRITE(38,*) H(1,1), H(1,2), H(1,3)
WRITE(38,*) H(2,1), H(2,2), H(2,3)
WRITE(38,*) H(3,1), H(3,2), H(3,3)
WRITE(38,*) "PRIMCOORD"
WRITE(38,*) 2*NP1, ' 1'
!
!
!
400 CONTINUE
!Calculate displacements and write them
DO i=1,NP1
  file = firstxyz
  READ(30,*,ERR=800,END=800) smass, x1, y1, z1
  file = secondxyz
  READ(31,*,ERR=800,END=800) smass, x2, y2, z2
  !CALL ATOMMASS(temp,smass)
  x21 = x2-x1
  y21 = y2-y1
  z21 = z2-z1
  !
  !If an atom crossed the border of the box and appeared
  !at the other end of the box because of PBC, the displacement
  !vector will be huge: we have to correct that
  IF(z21>H3/2.d0) THEN
    x21=x21-H(3,1)
    y21=y21-H(3,2)
    z21=z21-H(3,3)
  ENDIF
  IF(z21<-H3/2.d0) THEN
    x21=x21+H(3,1)
    y21=y21+H(3,2)
    z21=z21+H(3,3)
  ENDIF
  !
  IF(y21>H2/2.d0) THEN
    x21=x21-H(2,1)
    y21=y21-H(2,2)
  ENDIF
  IF(y21<-H2/2.d0) THEN
    x21=x21+H(2,1)
    y21=y21+H(2,2)
  ENDIF
  !
  IF(x21>H1/2.d0)  x21=x21-H(1,1)
  IF(x21<-H1/2.d0) x21=x21+H(1,1)
  !
  CALL ATOMSPIECES(smass,spieces)
  ! xyz_diff.xyz
  WRITE(35,415) spieces, x1, y1, z1, x21, y21, z21
  ! xyz_diff.norm
  norm = DSQRT( x21*x21 + y21*y21 + z21*z21 )
  WRITE(36,416) i, spieces, norm
  ! xyz_diff.xsf
  WRITE(37,417) INT(smass), x1, y1, z1, x21, y21, z21
  ! xyz_both.xsf
  WRITE(38,418) INT(smass), x1, y1, z1
  WRITE(38,418) INT(smass), x2, y2, z2
  !
  !We store norm of displacements in a table
  !for later statistics calculations (450)
  stattable(i,1) = smass
  stattable(i,2) = norm
ENDDO
415 FORMAT(a3,2X,6(f12.6,2X))
416 FORMAT(i6,X,a3,X,f12.6)
417 FORMAT(i3,2X,6(f12.6,2X))
418 FORMAT(i3,2X,3(f12.6,2X))
!
!Copy extended info at end of  xyzvec.xyz
DO i=1,12
  READ(30,'(a80)',ERR=430,END=430) temp
  WRITE(35,*) TRIM(ADJUSTL(temp))
ENDDO
!
430 CONTINUE
CLOSE(30)
CLOSE(31)
CLOSE(35)
CLOSE(36)
CLOSE(37)
CLOSE(38)
msg = '..> I wrote the file: xyz_diff.xyz'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '..> I wrote the file: xyz_diff.norm'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '..> I wrote the file: xyz_diff.xsf'
CALL DISPLAY_MSG(verbosity,msg,logfile)
msg = '..> I wrote the file: xyz_both.xsf'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
450 CONTINUE
!Calculate the statistics of displacements
OPEN(UNIT=39,FILE='xyz_stat.txt',FORM='FORMATTED',STATUS='UNKNOWN')
WRITE(39,*) 'STATISTICS ON THE DISPLACEMENTS'
WRITE(39,*) 'All values are in angstroms'
WRITE(39,*) ''
WRITE(39,*) 'Sp. = atomic Spieces (TO=Total statistics on all atoms)'
WRITE(39,*) 'm = Minimum displacement'
WRITE(39,*) 'M = Maximum displacement'
WRITE(39,*) 'A = Average displacement'
WRITE(39,*) 'D = Average absolute deviation'
WRITE(39,*) 'S = Standard deviation'
WRITE(39,*) ''
WRITE(39,*) 'Sp. |     m      |     M      |     A      |     D      |     S'
WRITE(39,*) '----+------------+------------+------------+------------+------------'
!
!Statistics on individual spieces
smass = 0.d0
spi_sp(:) = 0.d0
spi_NP(:) = 0
!Find total number of particles for each spieces
!and store it into spi_NP(:)
!Store atomic masses in spi_sp(:)
j = 0
DO i=1,NP1
  IF(stattable(i,1).NE.smass) THEN
    j = j+1
    smass = stattable(i,1)
    spi_sp(j) = smass
  ENDIF
  spi_NP(j) = spi_NP(j) + 1
ENDDO
!Now j is the number of distinct spieces
!For each spieces we calculate the statistics on the norm of displacements
DO i=1,j
  WRITE(msg,*) 'debug --> spieces, NP: ', spi_sp(i), spi_NP(i)
  CALL DISPLAY_MSG(verbosity,msg,logfile)
  !Create a table to store norm of displacements
  ALLOCATE( spi_table(spi_NP(i)) )
  spi_table(:) = 0.d0
  !For all atoms of this spieces, place norm of displ. in the table
  k = 0
  DO l=1,NP1
    IF( stattable(l,1) == spi_sp(i) ) THEN
      k = k+1
      spi_table(k) = stattable(l,2)
    ENDIF
  ENDDO
  CALL ATOMSPIECES(spi_sp(i),spieces)
  !Calculate statistics on the norm of displ.
  CALL DO_STATS( spi_table(:),stat_min,stat_M,stat_A,stat_D,stat_S )
  !Write the stats in the file
  WRITE(39,460) spieces, '|', stat_min, '|', stat_M, '|', &
       &         stat_A, '|',stat_D, '|', stat_S
  !Don't forget to deallocate the table for next loop
  DEALLOCATE(spi_table)
ENDDO
!
!Total statistics on all atoms
CALL DO_STATS( stattable(:,2),stat_min,stat_M,stat_A,stat_D,stat_S )
WRITE(39,*) '----+------------+------------+------------+------------+------------'
WRITE(39,460) 'TO', '|', stat_min, '|', stat_M, '|', stat_A, '|', stat_D, '|', stat_S
460 FORMAT(X,a2,2X,a1,X,5(f10.4,X,a1,X))
!
CLOSE(39)
msg = '..> I wrote the file: xyz_stat.txt'
CALL DISPLAY_MSG(verbosity,msg,logfile)
!
!
!
500 CONTINUE
msg = '..> Differencial displacements were computed.'
CALL DISPLAY_MSG(verbosity,msg,logfile)
GOTO 1000
!
!
!
800 CONTINUE
msg = 'X!X ERROR while reading the file '//TRIM(file)
CALL DISPLAY_MSG(3,msg,logfile)
WRITE(msg,*) '    This apparently occured while reading atom # ', i
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
GOTO 1000
!
!
850 CONTINUE
msg = 'X!X ERROR: the two systems do not have the same base vectors.'
CALL DISPLAY_MSG(3,msg,logfile)
nerr = nerr+1
GOTO 1000
!
!
!
1000 CONTINUE
IF(ALLOCATED(stattable)) DEALLOCATE(stattable)
!
!
END SUBROUTINE DIFF_XYZ
!
!
END MODULE xyz_difference
